#ifndef __TYPES_H__
#define __TYPES_H__

typedef unsigned int    u32;
typedef unsigned short  u16;
typedef unsigned char    u8;
typedef int             s32;
typedef short           s16;
typedef char             s8;

//
// Structures used to manage Ethernet communication
//
typedef struct {
    u8 mac1;
    u8 mac2;
    u8 mac3;
    u8 mac4;
    u8 mac5;
    u8 mac6;
} EthernetAddress;

//
// IPV4 simple address structure
//
typedef struct {
    u8 ip1;
    u8 ip2;
    u8 ip3;
    u8 ip4;
} InternetAddress;

// Ethernet header
typedef __packed struct 
{
    u8  dst[6];
    u8  src[6];
    u16 protocol;
} etherHdr;

// Ethernet protocol types
#define ETHERTYPE_IP   0x0800
#define ETHERTYPE_ARP  0x0806
#define ETHERTYPE_RARP 0x8035

// ARP packet with ethernet header (see RFC 826 - ARP)
typedef __packed struct 
{
    u16 hwType;			//2		//0
    u16 prType;			//2		//2
    u8  hwLength;		//1		//4
    u8  prLength;		//1		//5
    u16 operation;	//2				//6
    u8  senderHw[6];	//6			//8
    u32  senderIp;		//4				//14
    u8  targetHw[6];	//6			//18
    u32  targetIp;		//4				//24
} ARP_Hdr;

// ARP operation defines
#define ARP_REQUEST    1
#define ARP_REPLY      2
#define RARP_REQUEST   3
#define RARP_REPLY     4

#define ARP_ETHER      1

/**
 * UDP header based on RFC768
 */
typedef struct udphdr{
    u16 srcPort;		/* source port */
    u16 dstPort;		/* destination port */
    u16 length;
    u16 checksum;
} UDPHdr;

typedef struct 
{
    EthernetAddress DestAdr;
    EthernetAddress SourceAdr;
    u16 FrameType;
} EthHdr;

/*
 * Internet address (a structure for historical reasons)
 */
__packed struct in_addr {
	u32 s_addr;
};

typedef __packed struct ip
{
	u32	ip_hl:4,				/* header length */
		ip_v:4;						/* version */
	u8	ip_tos;					/* type of service */
	u16	ip_len;					/* total length */
	u16	ip_id;					/* identification */
	u16	ip_off;					/* fragment offset field */
#define	IP_DF 0x4000				/* dont fragment flag */
#define	IP_MF 0x2000				/* more fragments flag */
#define	IP_OFFMASK 0x1fff			/* mask for fragmenting bits */
	u8	ip_ttl;					/* time to live */
	u8	ip_p;					/* protocol */
	u16	ip_sum;					/* checksum */
	struct	in_addr ip_src,ip_dst;	/* source and dest address */
} IPHdr;

/*
 * Pseudo header for UDP checksum
 */
typedef __packed struct _ippseudo {
	struct	in_addr ip_src;		/* source internet address */
	struct	in_addr ip_dst;		/* destination internet address */
	u8	ip_zero;		/* 0 byte */
	u8	ip_pr;			/* protocol */
        u16	ip_sum;		/* udplength */
} PseudoIPHdr;


typedef struct _bootp {
    u8   op;             /* packet opcode type */
    u8   htype;          /* hardware addr type */
    u8   hlen;           /* hardware addr length */
    u8   hops;           /* gateway hops */
    s32   xid;            /* transaction ID */
    u16  secs;           /* seconds since boot began */
    u16  unused;
    struct in_addr  ciaddr; /* client IP address */
    struct in_addr  yiaddr; /* 'your' IP address */
    struct in_addr  siaddr; /* server IP address */
    struct in_addr  giaddr; /* gateway IP address */
    u8   chaddr[16];     /* client hardware address */
    u8    sname[64];      /* server host name */
    u8    file[128];      /* boot file name */
    u8   vend[64];       /* vendor-specific area */
} BOOTP_Hdr;

/** Timeout waiting for response from DHCP server in seconds */
#define DHCP_TIMEOUT 3
/** Number of retries before we give up */
#define DHCP_MAX_RETRYS 4

/** Magic number in DHCP packet */
#define DHCP_MAGIC 0x63825363

typedef enum {
	DHCPS_BEGIN,
	DHCPS_DISCOVER,
	DHCPS_REQUEST,
	DHCPS_OK,
	DHCPS_ABORT,
	DHCPS_TIMEOUT
} DHCP_STATE;

typedef enum {
	DHCPDISCOVER=1,
	DHCPOFFER=2,
	DHCPREQUEST=3,
	DHCPDECLINE=4,
	DHCPACK=5,
	DHCPNAK=6,
	DHCPRELEASE=7,
	DHCPINFORM=8,
	DHCPERROR=-1
} DHCP_MESSAGE_TYPE;

typedef struct {
	u8 op;
	u8 htype;
	u8 hlen;
	u8 hops;
	u32 xid;
	u16 secs;
	u16 flags;
	u32 ciaddr;
	u32 yiaddr;
	u32 siaddr;
	u32 giaddr;
	u8 chaddr[16];
	u8 sname[64];
	u8 file[128];
	u32 magic;
} DHCP_Hdr;

typedef struct PKHEADER
{ 
    u16 id; 
    u16 cmd; 
    u16 seq; 
    u16 off; 
    u16 seg; 
    u16 len;  
} PkHeader; 

typedef enum
{
    APCTL_OK = 1,
    APCTL_BUF_TOO_SMALL,
    APCTL_BAD_CMD,
    APCTL_CMD_FAILED,
    APCTL_TIMEOUT,
    APCTL_ERROR
}APCMD_STATUS;

typedef struct _APICMDBUF
{
    u16 cmdCode;
    u16 seqNum;
    u16 cmdStatus;
    u16 cmdBodyLen;
    u8 *cmdBody;
}
APICMDBUF;

#if 0
typedef struct {
	u32 my_ip;
	u32 dhcp_server;
	u32 router;
	u32 subnet_mask;
	u32 dns_count;
	u32 dns_server[2];
	u32 xid;
	u32 lease_time;
	u32 renewal_time;
	u32 rebinding_time;
	s32 err;
	s32 s;
	DHCP_STATE state;
	DHCP_HEADER h;
	DHCP_MESSAGE_TYPE lastMessageType;
} DHCP_Hdr;
#endif

typedef enum {
	DHCP_BOOTREQUEST=1,
	DHCP_BOOTREPLY=2
} DHCP_OP;

typedef enum {
	DHCP_ETHERNET=1
} DHCP_HTYPE;

/**
 * DHCP Option codes, courtesy of RFC2132
 */
typedef enum {
//    Code
//   +-----+
//   |  0  |
//   +-----+
	DHCP_OPTION_PAD=0,
//    Code   Len        Subnet Mask
//   +-----+-----+-----+-----+-----+-----+
//   |  1  |  4  |  m1 |  m2 |  m3 |  m4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_SUBNET_MASK=1,

//    Code   Len        Time Offset
//   +-----+-----+-----+-----+-----+-----+
//   |  2  |  4  |  n1 |  n2 |  n3 |  n4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_TIME_OFFSET=2,

//    Code   Len         Address 1               Address 2
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  3  |  n  |  a1 |  a2 |  a3 |  a4 |  a1 |  a2 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_ROUTER=3,

//    Code   Len         Address 1               Address 2
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  4  |  n  |  a1 |  a2 |  a3 |  a4 |  a1 |  a2 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_TIME_SERVER=4,

//    Code   Len         Address 1               Address 2
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  5  |  n  |  a1 |  a2 |  a3 |  a4 |  a1 |  a2 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_NAME_SERVER=5,

//    Code   Len         Address 1               Address 2
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  6  |  n  |  a1 |  a2 |  a3 |  a4 |  a1 |  a2 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_DNS_SERVER=6,

//    Code   Len         Address 1               Address 2
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  7  |  n  |  a1 |  a2 |  a3 |  a4 |  a1 |  a2 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_LOG_SERVER=7,

//    Code   Len         Address 1               Address 2
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  8  |  n  |  a1 |  a2 |  a3 |  a4 |  a1 |  a2 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_COOKIE_SERVER=8,

//    Code   Len         Address 1               Address 2
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  9  |  n  |  a1 |  a2 |  a3 |  a4 |  a1 |  a2 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_LPR_SERVER=9,

//    Code   Len                 Host Name
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
//   |  12 |  n  |  h1 |  h2 |  h3 |  h4 |  h5 |  h6 |  ...
//   +-----+-----+-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_HOST_NAME=12,

//    Code   Len      Size
//   +-----+-----+-----+-----+
//   |  13 |  2  |  s1 |  s2 |
//   +-----+-----+-----+-----+
	DHCP_OPTION_BOOT_FILE_SIZE=13,

//    Code   Len        Domain Name
//   +-----+-----+-----+-----+-----+-----+--
//   |  15 |  n  |  d1 |  d2 |  d3 |  d4 |  ...
//   +-----+-----+-----+-----+-----+-----+--
	DHCP_OPTION_DOMAIN_NAME=15,

//    Code   Len     Address   
//   +-----+-----+-----+-----+-----+-----+
//   |  28 |  4  |  a1 |  a2 |  a3 |  a4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_BROADCAST_ADDRESS=28, 

//    Code   Len     Address   
//   +-----+-----+-----+-----+-----+-----+
//   |  44 |  4  |  a1 |  a2 |  a3 |  a4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_NETBIOS_OVER_TCP=44, 

//    Code   Len          Address
//   +-----+-----+-----+-----+-----+-----+
//   |  50 |  4  |  a1 |  a2 |  a3 |  a4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_REQUESTED_IP_ADDRESS=50,
	
//    Code   Len         Lease Time
//   +-----+-----+-----+-----+-----+-----+
//   |  51 |  4  |  t1 |  t2 |  t3 |  t4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_LEASE_TIME=51,

//           Value   Message Type
//           -----   ------------
//             1     DHCPDISCOVER
//             2     DHCPOFFER
//             3     DHCPREQUEST
//             4     DHCPDECLINE
//             5     DHCPACK
//             6     DHCPNAK
//             7     DHCPRELEASE
//             8     DHCPINFORM
//
//    Code   Len  Type
//   +-----+-----+-----+
//   |  53 |  1  | 1-9 |
//   +-----+-----+-----+
	DHCP_OPTION_MESSAGE_TYPE=53,

//    Code   Len            Address
//   +-----+-----+-----+-----+-----+-----+
//   |  54 |  4  |  a1 |  a2 |  a3 |  a4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_SERVER_IDENTIFIER=54,

//    Code   L7en   Option Codes
//   +-----+-----+-----+-----+---
//   |  55 |  n  |  c1 |  c2 | ...
//   +-----+-----+-----+-----+---
	DHCP_OPTION_PARAMETER_LIST_REQUEST=55,

//    Code   Len   Msg Length
//   +-----+-----+-----+-----+
//   |  57 |  2  |  l1 |  l2 |
//   +-----+-----+-----+-----+
	DHCP_OPTION_MAX_DHCP_MSG_SIZE=57, 

//    Code   Len         T1 Interval
//   +-----+-----+-----+-----+-----+-----+
//   |  58 |  4  |  t1 |  t2 |  t3 |  t4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_RENEWAL_TIME=58,

//    Code   Len         T2 Interval
//   +-----+-----+-----+-----+-----+-----+
//   |  59 |  4  |  t1 |  t2 |  t3 |  t4 |
//   +-----+-----+-----+-----+-----+-----+
	DHCP_OPTION_REBINDING_TIME=59,

//   Code   Len   Type  Client-Identifier
//   +-----+-----+-----+-----+-----+---
//   |  61 |  n  |  t1 |  i1 |  i2 | ...
//   +-----+-----+-----+-----+-----+---
	DHCP_OPTION_CLIENT_IDENTIFIER=61,

//    Code
//   +-----+
//   | 255 |
//   +-----+
	DHCP_OPTION_END=255
} DHCP_OPTION_TAG;

#endif
