/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * Purpose:
 *    Low-level helper code to setup the uart for data transmission.
 *
 * Public Procedures:
 *    getchar                 Read a char from the 16550 UART block. 
 *                            When one is avail.
 *    putchar                 Write a u8 to the 16550.
 *    setup_uart              Prepare the 16550 for 38400 NP 1stop also
 *                            set the pin mux to UART from USB
 *
 * Private Procedures:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

//=============================================================================
//                               INCLUDE FILES
//=============================================================================
#include <stdlib.h>
#include <stdio.h>
#include "uart.h"

//=============================================================================
//                         CODED PUBLIC PROCEDURES
//=============================================================================
void uart_printf(const char * fmt, ...)
{
    unsigned char buf[128];
    va_list ap;
    va_start(ap, fmt);
    
    vsprintf(buf, fmt, ap);
    va_end(ap);
    myputs(buf);
    myputchar('\r');
    return;
}

/******************************************************************************/
void myputs(unsigned char *c)
{
    int i=0;

    while(*c)
    {
        myputchar(*c);
        c++;
        if ((i++)>128) {
            myputchar('\n');myputchar('\r');
            break;
        } 
    }
}

/******************************************************************************
 *
 * Name: getchar
 *
 * Description:
 *   Return the next char available on UART receive. If no char is
 *   ready, it will wait. Should no char still come after a timeout
 *   passes, then return back to the   uart_boot() code and NAK the
 *   packet. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *   Must be run with cache off or NAK_TIMEOUT loop will not be 10sec.
 *   Requires ARM ADS1.2 Thumb compiler. Again, timeout loop sensitivity.
 *
 * Arguments:
 *   Arg1( u16 ): 16 bits of data to appear at the pins.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern int mygetchar( unsigned char *pu8c )
{
    int s32counter = 0;

    while(!(REG32(LINE_STATUS) & DATA_READY))
    {
        s32counter++;
        if((s32counter>>16) >= NAK_TIMEOUT_VAL) return FAIL;
    }
    *pu8c = REG32(Rx_BUFFER_REG);
    return OK;
}
/******************************************************************************
 *
 * Name: putchar
 *
 * Description:
 *   Put a char into the UART transmit buffer. Wait until that buffer
 *   is ready. No timeout here, assume we can always be ready to send
 *   after a wait. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *
 * Arguments:
 *   Arg1( u8 ): 8 bits of data to be placed in the 16550 xmit buffer.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void myputchar( unsigned char u8c )
{
    while(!(REG32(LINE_STATUS) & Tx_EMPTY));    // Tx_EMPTY is 1 when 
	                                        // empty
    REG16(Tx_HOLDING_REG) = u8c;
}

/******************************************************************************
 *
 * Name: setup_uart
 *
 * Description:
 *   Place the 16550 into a useable state for us. This means the the
 *   baud rate needs to be 38400, no parity, one stop bit. Also the
 *   pad mux needs to get flipped from the default of USB to UART. 
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
void setup_uart( int rate )
{
    unsigned int baudrate_dividor;
    unsigned int pi_pad_ctrl;

    pi_pad_ctrl          = REG32(PI_PAD_CTRL);
    REG32(PI_PAD_CTRL)   = pi_pad_ctrl & (~bit(11)); // De-assert the mux for USB/UART --> UART.

    REG32(LINE_CONTROL)  = DLAB; // enable DIVISOR_LOW and DIVISOR_HIGH registers. 

    baudrate_dividor     = SERIAL_SPEED / 16 / rate + 0.5;
    REG32(DIVISOR_LOW)   = (unsigned char)(baudrate_dividor  & 0xff);
    REG32(DIVISOR_HIGH)  = (unsigned char)(baudrate_dividor >> 8   );

    REG32(LINE_CONTROL)  = ((EIGHT_BIT) & ~(TWO_STOP_BITS) & ~(PARITY_ENABLE) & ~(DLAB));

    REG32(FIFO_CONTROL)  = 7; // enable FIFO, reset FIFO.
}
