/*******************************************************************
*
*  DESCRIPTION: EEprom driver routines for one type of SPI device
*
*  AUTHOR: hrainnie@marvell.com (heavily borrowing cvu's oemflash code)
*
*  HISTORY:
*  06nov03,hrainnie,created
*
*  (C) Marvell Semiconductor, Inc. 2003
*******************************************************************/

/** include files **/
#ifdef ECOS
#include <cyg/kernel/kapi.h>
#endif
#include "wltypes.h"
#include "apcfg.h"
#include "IEEE_types.h"
#include "wl_mib.h"
#include "wl_hal.h"
#include "flashdrvapi.h"
#include "os_if.h"
#include "usrcfg.h"

typedef unsigned int    u32;
typedef unsigned short  u16;
typedef unsigned char    u8;
typedef int             s32;
typedef short           s16;
typedef char             s8;

#define REG32(x) (*(volatile u32 *)(x))
#define REG8(x) (*(volatile u8 *)(x))
#define REG16(x) (*(volatile u16 *)(x))


extern int apctlIntDisable(void);
extern int apctlIntEnable(int);
extern void *malloc (size_t size);
extern void free (void *ptr);
extern void WatchdogDisable (void);
extern void WatchdogEnable (void);


/** local definitions **/

#define _REG16(x) (*(volatile unsigned short int *)(x))

/* default settings */

/* AP eeprom memory Map on 32k eeprom part:
     Sector     Description
     ------     -----------
       0..27k   Boot loader
       28..29k  Factory and user configuration data (1KB each):
                   Offset 0..1023: factory settings
                   Offset 1024..2047: user settings
       30..31k  unused.
       
*/

#define OFFSET_FACTORY_CONFIG 0x7000
#define SIZE_FACTORY_CONFIG   1024

#define SECTOR_USER_CONFIG    OFFSET_FACTORY_CONFIG
#define OFFSET_USER_CONFIG    (OFFSET_FACTORY_CONFIG + SIZE_FACTORY_CONFIG)
#define SIZE_USER_CONFIG      SIZE_FACTORY_CONFIG

#define OFFSET_MFG	      (OFFSET_FACTORY_CONFIG + 0x370)
#define OFFSET_CAL        (OFFSET_FACTORY_CONFIG + 0x370)
#define SIZE_MFG          0x90  
/*----- Types and Constants Definition ----------------------*/
#define REG32(x) (*(volatile u32 *)(x))
#define REG8(x) (*(volatile u8 *)(x))
#define REG16(x) (*(volatile u16 *)(x))

u16 g_u16AddrLen;

#define EAROM_SIZE (32*1024)

#define CON4 (1 << 4)

#define SPI_WREN 0x420
#define SPI_ERASE 0x420
#define SPI_WRDI 0x410
#define SPI_READ_ADDR_DATA(addr_len, data_len)	(0x402 | ((((addr_len) + (data_len)) - 1) << 6))
#define SPI_WRITE_ADDR_DATA(addr_len, data_len)	(0x401 | ((((addr_len) + (data_len)) - 1) << 6))
#define SPI_RSTATUS		0x408	// spi start, len 0, read status

#define READ		          0x3   // Command that the eeprom uses for read
#define RDSR 		5
#define WREN 		6
#define WRDI 		4
#define PP   		2
#define BULK_ERASE 0xC7

/** external functions **/

/** external data **/

/** public data **/

/** private data **/

static int flashDrvInitialized = FALSE;
#ifdef MUTEX_LOCK_EEPROM
static cyg_mutex_t writeFlash_mutex;
#endif

static int writing_in_progress = 0;
/* A buffer to store the factory settings */
static char mFactorySettings[SIZEOF_MIB_USER_AP];
static char mFactorySettingRead;
/** private functions **/

u32 result1=0;
u32 result2=0;
u32 result3=0;

void
spi_access()
{
    REG32(0x8000c100) = 0x8; // clock rate
    REG32(0x8000c104) = 0x82; // GCTRL. (bits 7:6-> 10 == SPI mode)
}
void
spi_init( u16 u16strap )
{
    REG32(0x8000c000) = 1; // soft reset p_rst_
    REG32(0x8000c004) = 0; // cs_pclk2 = gclk
    spi_access();
    if(u16strap & CON4) { /* Check CON[4] */
	g_u16AddrLen = 2;
    } else {
	g_u16AddrLen = 3;
    }
}

void
spi_doneclr()
{
    while(REG32(0x8000c108) & 0x400);
 
}
void
spi_wren()
{
    REG32(0x8000c108) = SPI_WREN;
    REG32(0x8000c124) = WREN;
    spi_doneclr();
}
void
spi_wrdi()
{
    REG32(0x8000c108) = SPI_WRDI;
    REG32(0x8000c124) = WRDI;
    spi_doneclr();
}
u32
spi_rdsr()
{
    u32 result;
    REG32(0x8000c108) = SPI_RSTATUS;
    REG32(0x8000c124) = RDSR;
    result = REG32(0x8000c124);
    spi_doneclr();
    return result;
}
s32
spi_write2(u32 u32addr, u16 u16data)
{
    for(;;);
}

s32
spi_write4(u32 u32addr, u32 u32data)
{
    spi_access();
    spi_wren();
    REG32(0x8000c108) = SPI_WRITE_ADDR_DATA(g_u16AddrLen, 4);
    REG32(0x8000c124) = PP;
    if(g_u16AddrLen == 3) {
	REG32(0x8000c124) = (u32addr >> 16) & 0xff;
    }
    REG32(0x8000c124) = (u32addr >>  8) & 0xff;
    REG32(0x8000c124) = u32addr & 0xff;
    REG32(0x8000c124) = (u32data >> 24) & 0xff;
    REG32(0x8000c124) = (u32data >> 16) & 0xff;
    REG32(0x8000c124) = (u32data >>  8) & 0xff;
    REG32(0x8000c124) = u32data & 0xff;
    spi_doneclr();
    while(spi_rdsr() & 1);
    spi_wrdi();
    return TRUE;
}
// In the AP library there are references to this symbol that I have deprecated
int flashWriteWord(int offset, unsigned short dat) __attribute((weak, alias("spi_write2")));

u32
spi_read4(u32 u32addr)
{
    s16 i;
    u32 result = 0;
    spi_access();
    REG32(0x8000c108) = SPI_READ_ADDR_DATA(g_u16AddrLen, 4); // 24bit addr + 4 return
    REG32(0x8000c124) = READ; // Tell the part what we want to do
    if(g_u16AddrLen == 3) {
	REG32(0x8000c124) = (u32addr>>16) & 0xff;  // addr0 [23:16]
    }
    REG32(0x8000c124) = (u32addr>>8) & 0xff;   // addr1 [16:8]
    REG32(0x8000c124) = u32addr & 0xff;        // addr2 [7:0]
    // Loop to grab all 4 bytes
    for(i=0;i < 4;i++) {
	result <<= 8;
	result |= (REG32(0x8000c124) & 0xff);
    }
    spi_doneclr();

//    result = WORD_SWAP(result);
    return result;
}

/******************************************************************************
 *
 * Name: EraseMfgData
 *
 * Description:
 *    Routine to erase block of memory from the flash
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    Arg1 (i  ): starting block
 *    Arg2 (i  ): number of block
 *
 * Return Value:
 *    TRUE is success.
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
int EraseMfgData (int sblk, int nblks)
{
    unsigned int *src;
    unsigned int *dest;
    unsigned long nbytes;
    unsigned long offset;
    char *ptr;
    int status = TRUE;

    ptr = malloc(SIZE_MFG);
    if (ptr)
    {
        WatchdogDisable();
        memset(ptr, 0xff, SIZE_MFG);

        src = (unsigned int *)ptr;
        nbytes =SIZE_MFG;

        offset = OFFSET_MFG;
        while ((nbytes >= sizeof(UINT32)) && (status == TRUE))
        {
            status = spi_write4(offset, *src);
            ++src;
            offset += sizeof(UINT32);
            nbytes -= sizeof(UINT32);
        }
        free(ptr);
        WatchdogEnable();
    }

    return (status);
}

/**
 * Erase a sector
 * 
 * @param sectorNum The sector number
 * @return TRUE on success
 *         FALSE on failure
 */
static int
EraseFlashSector(int sectorNum)
{
    return TRUE;
}

/**
 * Read factory settings from flash
 * 
 * @param factoryData
 *               Pointer to the buffer to receive the results
 * @return TRUE: success
 *         FALSE: failure
 */
static int
AP21_ReadFactorySettings(mib_AP_t *factoryData) // 2003/05/29, fred chang
{
    int nbytes;
    unsigned short src;
    unsigned int *dest;

    if (!flashDrvInitialized || (factoryData == NULL))
    {
        return FALSE;
    }

    nbytes = SIZEOF_MIB_AP; 
    if(nbytes > SIZE_FACTORY_CONFIG - SIZE_MFG)
        nbytes = SIZE_FACTORY_CONFIG - SIZE_MFG;
    src = OFFSET_FACTORY_CONFIG;
    dest = (unsigned int *)factoryData;

    while (nbytes>=sizeof(UINT32))
    {
	*dest++ = spi_read4(src);
        nbytes -= sizeof(UINT32);
	src += sizeof(UINT32);
    }

    return TRUE;
}

/**
 * Read user settings from flash
 * 
 * @param factoryData
 *               Pointer to the buffer to receive the results
 * @return TRUE: success
 *         FALSE: failure
 */
static int
AP21_ReadUserSettings(mib_AP_t *userData) // 2003/05/29, fred chang
{
    int nbytes;
    unsigned short src;
    unsigned int *dest;

    if (!flashDrvInitialized || (userData == NULL))
    {
        return FALSE;
    }
    if(!mFactorySettingRead){
#ifdef MUTEX_LOCK_EEPROM
    	cyg_mutex_lock( &writeFlash_mutex );
#endif
        nbytes = SIZEOF_MIB_USER_AP; 
    	src = OFFSET_USER_CONFIG;
        dest = (unsigned int *)mFactorySettings;

    	while (nbytes>=sizeof(UINT32))
    	{
			*dest++ = spi_read4(src);
        	nbytes -= sizeof(UINT32);
			src += sizeof(UINT32);
    	}
#ifdef MUTEX_LOCK_EEPROM
    	cyg_mutex_unlock( &writeFlash_mutex );
#endif
    }
    memcpy(userData, mFactorySettings, SIZEOF_MIB_USER_AP);
    return TRUE;
}


/**
 * Write user configuration to flash
 * 
 * @param userData Pointer to user data
 * @return TRUE: success
 *         FALSE: failure
 */
static int
AP21_WriteUserSettings(mib_AP_t *userData) 
{
    int nbytes;
    unsigned int *src, *src1;
    int offset;

    if (!flashDrvInitialized || (userData == NULL))
    {
        return FALSE;
    }
    if (writing_in_progress)
    {
        memcpy(mFactorySettings, userData, SIZEOF_MIB_USER_AP);
        return FALSE;
    }
#ifdef MUTEX_LOCK_EEPROM
    cyg_mutex_lock( &writeFlash_mutex );
#endif
    WatchdogDisable();

    os_EnterCriticalSection;

    /* Now write the user data */
    nbytes = SIZEOF_MIB_USER_AP; // 2003/05/29, fred chang
    src = (unsigned int *)userData;
    src1= (unsigned int *)mFactorySettings;
    offset = OFFSET_USER_CONFIG;

    while (nbytes>=sizeof(UINT32))
    {
        if(*src1 != *src){
            writing_in_progress = 1;
            *src1 = *src;
            spi_write4(offset, *src);
        }
        ++src;
        ++src1;
        offset += sizeof(UINT32);
        nbytes -= sizeof(UINT32);
    }
    writing_in_progress = 0;
    os_ExitCriticalSection;
    WatchdogEnable();
#ifdef MUTEX_LOCK_EEPROM
    cyg_mutex_unlock( &writeFlash_mutex );
#endif    
    return TRUE;
}

/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
static int
AP21_ResetToFactorySettings(void)
{
	
    int rc;

    rc = FALSE;
    memset(mFactorySettings, 0xff, SIZEOF_MIB_USER_AP); 

    /* Read the factory default configuration from flash memory */
    if (AP21_ReadFactorySettings((mib_AP_t *)mFactorySettings)) // 2003/05/29, fred chang
    {
        rc = AP21_WriteUserSettings((mib_AP_t *)mFactorySettings); // 2003/05/29, fred chang
    }

    return rc;
}
static int
AP21_Check_Image_Integrality(char *pData, unsigned long len)
{
    // eeprom..???
    return TRUE;
}
#define ENABLE_BACKUP_CODE 1 
/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
#if 1//bt_test
extern int
#else
static int
#endif//bt_test
AP21_UpgradeFirmware(char *pData, unsigned long len)
{
    return TRUE;
}

/*
 * called via the API
 */
static int
AP21_GetEtherAddr(unsigned char *etherAddr)
{
    unsigned result;
    if(etherAddr == NULL) {
	return FALSE;
    }
    result = spi_read4(0x73b0);
    if(result != 0xffffffff)
    {
        memcpy(&etherAddr[0], &result, 4);
        result = spi_read4(0x73b4);
        memcpy(&etherAddr[4], &result, 2);
        return TRUE;
    }
    result = spi_read4(0xc);
    result = WORD_SWAP(result);
    memcpy(&etherAddr[2], &result, 4);
    result = spi_read4(0x10);
    result = WORD_SWAP(result);
    memcpy(etherAddr, ((unsigned char *)&result)+2, 2);
    return TRUE;
}

MFG_CAL_DATA calDataInMem;
static void *
AP21_GetCalData(void)
{
    int nbytes;
    unsigned short src;
    unsigned int *dest;
    static int calDataInMemInit = 0;

    if (!flashDrvInitialized )
    {
        return FALSE;
    }
    if(!calDataInMemInit)
    {
#ifdef MUTEX_LOCK_EEPROM
        cyg_mutex_lock( &writeFlash_mutex );
#endif
        nbytes = sizeof(MFG_CAL_DATA);
        src = OFFSET_CAL;
        dest = (unsigned int *)&calDataInMem;

        while (nbytes>=sizeof(UINT32))
        {
        *dest++ = spi_read4(src);
            nbytes -= sizeof(UINT32);
        src += sizeof(UINT32);
        }
        calDataInMemInit = 1;
#ifdef MUTEX_LOCK_EEPROM
        cyg_mutex_unlock( &writeFlash_mutex );
#endif
    }
    return  (void *)&calDataInMem;
}

/** public functions **/

// This has no meaning with a 32k eeprom (!)
extern int
ApUpgradeFirmware(char *pData, unsigned long len)
{
    return TRUE;
}
/**
 * User supplied function to initialize Flash driver struct
 * At startup time, the system will call this function to initialize
 * the flash driver.
 * 
 * @param apfdDriver_t
 *               Pointer to Flash Driver init struct.
 *               The initialized flash driver stuct is returned.
 */
void
apfdDriverInit(apfdDriver_t *apfdDrv)
{
    u16 strap;
    if (apfdDrv != NULL)
    {
        apfdDrv->WordMode = TRUE;
        apfdDrv->wrfunc.WrWord = spi_write2;
        apfdDrv->EraseSector = EraseFlashSector;
        apfdDrv->ReadFactorySettings = AP21_ReadFactorySettings;
        apfdDrv->ReadUserSettings = AP21_ReadUserSettings;
        apfdDrv->WriteUserSettings = AP21_WriteUserSettings;
        apfdDrv->ResetToFactorySettings = AP21_ResetToFactorySettings;
        apfdDrv->UpgradeFirmware = AP21_UpgradeFirmware;
        apfdDrv->MAC_Addr = AP21_GetEtherAddr;
        apfdDrv->CalData = AP21_GetCalData;
    }
    // Read the strap logic so we know how wide the eeprom is.
    strap = REG16(0x80002000);
    spi_init(strap);

    flashDrvInitialized = TRUE;

}

void WaitForFlashReady(void)
{
#ifdef MUTEX_LOCK_EEPROM
    cyg_mutex_lock( &writeFlash_mutex );
    cyg_mutex_unlock( &writeFlash_mutex );
#endif
    while(writing_in_progress)
    {
        os_TaskDelay(10);
    }
}
void
spi_check()
{
    result1 = spi_read4(0x7000);
}
