/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    oemflash.c
 * \brief   Flash related functions including FW upgrade
 *
 * $Revision: #13 $
 * $Date: 2005/11/28 $
 */

#ifdef ECOS
#include <cyg/kernel/kapi.h>
#endif
#include "wltypes.h"
#ifdef AP5X
#include "oem.h"
#endif
#include "apcfg.h"
#include "flashdrvapi.h"
#ifdef AP5X
#include "os_if.h"
#else
#include "osif.h"
#endif
#include "usrcfg.h"

#ifdef VOIP
// Increase VOIP_CONFIG_VERSION whenever VoIP config data structure changes
#define VOIP_CONFIG_VERSION	4
#endif

extern int apctlIntDisable(void);
extern int apctlIntEnable(int);
extern void *malloc (size_t size);
extern void free (void *ptr);
extern void WatchdogDisable (void);
extern void WatchdogEnable (void);

/** local definitions **/

/* Manufacturer code */
#define AMD	        0x01
#define HYUNDAI 	0xAD
#define MICRONIX 	0xC2
#define SST         0xBF

#define _REG16(x) (*(volatile unsigned short int *)(x))

/* default settings */
#define SIZEOF_FLASH_SECTOR (64 * 1024)
#define SECTOR_OFFSET(sector) (sector * SIZEOF_FLASH_SECTOR)

/* AP Flash memory Map on 1MB flash part:
     Sector     Description
     ------     -----------
       15       Boot loader
       14       Factory and user configuration data (1KB each):
                   Offset 0..1023: factory settings
                   Offset 1024..2047: user settings
     13..0      Firmware image
*/
#ifdef ECOS
#define SECTOR_FACTORY_CONFIG (0xfffe0000 - flashbase)/SIZEOF_FLASH_SECTOR
#else /* VxWorks settings for now to avoid conflict with armboot. */
#define SECTOR_FACTORY_CONFIG (0xfffd0000 - flashbase)/SIZEOF_FLASH_SECTOR
#endif
#define OFFSET_FACTORY_CONFIG (SECTOR_FACTORY_CONFIG * SIZEOF_FLASH_SECTOR)
#define SIZE_FACTORY_CONFIG   1024

#ifdef OS_VxW
static int factory_sector = 13;
#endif

#define SECTOR_USER_CONFIG    SECTOR_FACTORY_CONFIG
#define OFFSET_USER_CONFIG    (OFFSET_FACTORY_CONFIG + SIZE_FACTORY_CONFIG)
#define SIZE_USER_CONFIG      SIZE_FACTORY_CONFIG

/** external functions **/

/** external data **/

/** public data **/

/** private data **/
typedef struct FLASH_TYPE_t
{
    unsigned index;
    unsigned manu;
    unsigned addr1;
    unsigned addr2;
    unsigned val1;
}FLASH_TYPE_t;

#define MAX_VENDOR 4
FLASH_TYPE_t flash[] =
{
    {0, AMD, 0xAAA, 0x555, 0x0030},
    {1, HYUNDAI, 0xAAA, 0x555, 0x0030},
    {2, MICRONIX, 0xAAA, 0x555, 0x0030},
    {3, SST, 0xAAAA, 0x5554, 0x0050}
};
unsigned flashindex = 0;

unsigned int flashsize = 0;
unsigned int flashbase = 0xffff0000;
unsigned int flashNsect = 0;
unsigned int manu = 0;
unsigned int part = 0;

#ifdef VOIP
unsigned int voip_config_sector;
#endif

static int flashDrvInitialized = FALSE;
static int split_sector_kb = 0;

static int split_sector = -1;
#ifdef ECOS
static cyg_mutex_t writeFlash_mutex;
#endif

#define NUM_SUBSECTORS 4
static int split_sector_data[NUM_SUBSECTORS] = {
            0,
            0x8000,
            0xa000,
            0xc000,
        };

static int split[2][4] = {
                             { 16, 8, 8, 32},
                             { 32, 8, 8, 16}
                         };

/* A buffer to store the factory settings */
static char mFactorySettings[SIZEOF_MIB_USER_AP];
/** private functions **/

/**
 * Flash write or erase preamble for AMD part
 */
inline void
flashPreamble(void)
{
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;
    _REG16(flashbase | flash[flashindex].addr1) = 0x0080;
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;
}

/**
 * Build split sector configuration data 
 * 
 * @param ss
 */
void
makesplit(unsigned ss)
{
    int top = 0;
    int start = 0;
    int i;

    split_sector = ss;

    if (ss)
    {
        top = 1;
    }

    for (i = 0; i < 4; ++i)
    {
        split_sector_data[i] = start;
        start += split[top][i] * 1024;
    }
}

/**
 * Interrogate flash part for manufacturer info and part info
 */
void
flashInterrogate(void)
{
    int i;
    for ( i = 0; i<MAX_VENDOR; i++ ) {
        _REG16(flashbase) = 0xf0; /* reset */

        _REG16(flashbase | flash[i].addr1) = 0x00AA;  /* word address 555 */
        _REG16(flashbase | flash[i].addr2) = 0x0055;  /* word address 2AA */
        _REG16(flashbase | flash[i].addr1) = 0x0090;  /* word address 555 */
        manu = _REG16(flashbase);     /* Manu */
        part = _REG16(flashbase + 2); /* part ID. */
        _REG16(flashbase) = 0xf0; /* reset */
        if ( manu == flash[i].manu ) {
            flashindex = i;
            return;
        }
    }
    flashindex = 0;
}

/**
 * Erase a block of sectors
 *
 * @param sblk   Start sector number
 * @param nblks  Number of sectors
 */
void
EraseSectors(int sblk, int nblks)
{
    unsigned addr, start;
    int i;
    int j;

    if ( !flashDrvInitialized ) {
        return;
    }

    for ( i = sblk;i < (sblk + nblks);i++ ) {
        addr = flashbase + SECTOR_OFFSET(i);
        if ( i != split_sector ) 
        {
            flashPreamble();
            _REG16(addr) = flash[flashindex].val1;
            while ( _REG16(addr) != 0xFFFF );
        }
        else 
        {
            start = addr;
            if ( split_sector_kb ) 
            {
                for ( j=0; j<(SIZEOF_FLASH_SECTOR/(split_sector_kb *1024)); j++ ) {
                    addr = start + j * (split_sector_kb *1024); // 8kb
                    flashPreamble();
                    _REG16(addr) = flash[flashindex].val1;
                    while ( _REG16(addr) != 0xFFFF );
                }
            }
            else 
            {
                for ( j = 0; j < NUM_SUBSECTORS; j++ ) {
                    addr = start + split_sector_data[j];
                    flashPreamble();
                    _REG16(addr) = flash[flashindex].val1;
                    while ( _REG16(addr) != 0xFFFF );
                }
            } // if split 8kb
        } // if split
    } // for
    return; 
}

/**
 * Write a word (16 bit)
 *
 * @param offset The offset of the address relative to the start of the flash
 * @param value  The value to write
 * @return TRUE: success
 *         FALSE: failure
 */
int
flashWriteWord( int offset, unsigned short dat)
{

    unsigned addr = flashbase | offset;
    unsigned short tmp;

    if (!flashDrvInitialized)
    {
        return FALSE;
    }

    /*flags = apctlIntDisable(); */
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;  /* word address 555 */
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;  /* word address 2AA */
    _REG16(flashbase | flash[flashindex].addr1) = 0x00A0;  /* word address 555 */

    /* if flash is not erase we don't write (avoid infinit loop) */
    /* we can change a bit from 1 to 0 but not from 0 to 1 */
    tmp = _REG16 (addr);
    if ((~tmp & dat) != 0x00000000)
        return FALSE;   /* flash not erased */

    _REG16(addr) = dat;

    /* spin here until operation is done */
    while (_REG16(addr) != dat)
        ;


    return TRUE;
}

/******************************************************************************
 *
 * Name: EraseMfgData
 *
 * Description:
 *    Routine to erase block of memory from the flash
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    Arg1 (i  ): starting block
 *    Arg2 (i  ): number of block
 *
 * Return Value:
 *    TRUE is success.
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
int EraseMfgData (int sblk, int nblks)
{
    unsigned short *src;
    unsigned short *dest;
    unsigned long nbytes;
    unsigned long offset;
    char *ptr;
    int status = TRUE;

    ptr = malloc(64 * 1024);
    if (ptr)
    {
        WatchdogDisable();
        nbytes = 64 * 1024;
        src = (unsigned short *)(mfgDataAddress() & 0xffff0000L);
        dest = (unsigned short *)ptr;

        while (nbytes)
        {
            *dest++ = *src++;
            nbytes -= sizeof(UINT16);
        }
#ifndef FALCON
	#ifdef NEWCALDATA	
		/* As of now, only two locations are used for 8010,8030 cal data */
		/* The cal data initially may be empty, so autodetect can't be done */
		if (mfgDataAddress() == 0xfffff000)
		{
        	/* 4032 = (FC0)h = F000 - FC0 */
        	memset(ptr + (mfgDataAddress() & 0x0000ffffL), 0xff, 4024);
		}
	    else  /* compatibale with Rev. lower */
		{
	        memset(ptr + (mfgDataAddress() & 0x0000ffffL), 0xff, 696);
	    }
	#else
        memset(ptr + (mfgDataAddress() & 0x0000ffffL), 0xff, 696);
	#endif /* NEWCALDATA */
#else
        memset(ptr + (mfgDataAddress() & 0x0000ffffL), 0xff, 72);
#endif
        EraseSectors(sblk, nblks);
        src = (unsigned short *)ptr;
        nbytes = 64 * 1024;
        /*     offset = 0xffff0000; */
        offset = SECTOR_OFFSET (sblk) ;  /* so offset + flashbase = 0xFFFF0000 */
        while ((nbytes != 0) && (status == TRUE))
        {
            status = flashWriteWord(offset, *src);
            ++src;
            offset += sizeof(UINT16);
            nbytes -= sizeof(UINT16);
        }
        free(ptr);
        WatchdogEnable();
    }

    return (status);
}

/**
 * Erase a sector
 * 
 * @param sectorNum The sector number
 * @return TRUE on success
 *         FALSE on failure
 */
static int
EraseFlashSector(int sectorNum)
{
    if (!flashDrvInitialized || (sectorNum >= flashNsect))
    {
        return FALSE;
    }

    EraseSectors(sectorNum, 1);

    return TRUE;
}

/**
 * Read factory settings from flash
 * 
 * @param factoryData
 *               Pointer to the buffer to receive the results
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApReadFactorySettings(mib_AP_t *factoryData)
{
    int nbytes;
    unsigned short *src;
    unsigned short *dest;

    if (!flashDrvInitialized || (factoryData == NULL))
    {
        return FALSE;
    }

    nbytes = SIZEOF_MIB_AP;
    src = (unsigned short *)(flashbase + OFFSET_FACTORY_CONFIG);
    dest = (unsigned short *)factoryData;

    while (nbytes)
    {
        *dest++ = *src++;
        nbytes -= sizeof(UINT16);
    }

    return TRUE;
}

/**
 * Read user settings from flash
 * 
 * @param factoryData
 *               Pointer to the buffer to receive the results
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApReadUserSettings(mib_AP_t *userData)
{
    int nbytes;
    unsigned short *src;
    unsigned short *dest;

    if (!flashDrvInitialized || (userData == NULL))
    {
        return FALSE;
    }

#ifdef ECOS
    if(cyg_thread_self() != cyg_thread_idle_thread())
        cyg_mutex_lock( &writeFlash_mutex );
#endif

	nbytes = SIZEOF_MIB_USER_AP;
    src = (unsigned short *)(flashbase + OFFSET_USER_CONFIG);
    dest = (unsigned short *)userData;

    while (nbytes)
    {
        *dest++ = *src++;
        nbytes -= sizeof(UINT16);
    }

#ifdef ECOS
    if(cyg_thread_self() != cyg_thread_idle_thread())
        cyg_mutex_unlock( &writeFlash_mutex );
#endif

    return TRUE;
}


/**
 * Write user configuration to flash
 * 
 * @param userData Pointer to user data
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApWriteUserSettings(mib_AP_t *userData) /* 2003/05/29, fred chang */
{
    int nbytes;
    unsigned short *src;
    int offset;

    if (!flashDrvInitialized || (userData == NULL))
    {
        return FALSE;
    }

    /* We are going to erase the sector, so must save the factory settings */
    /* because it is on the same sector as the user data */
    ApReadFactorySettings((mib_AP_t *)mFactorySettings);

#ifdef ECOS
    if(cyg_thread_self() != cyg_thread_idle_thread())
        cyg_mutex_lock( &writeFlash_mutex );
#endif

    WatchdogDisable();
#ifdef ECOS
    if(cyg_thread_self() != cyg_thread_idle_thread())
#endif        
        os_EnterCriticalSection;
    
  	     EraseFlashSector(SECTOR_FACTORY_CONFIG);

    /* Now write the factory data */
    nbytes = SIZEOF_MIB_AP;
    src = (unsigned short *)mFactorySettings;
    offset = OFFSET_FACTORY_CONFIG;

    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }

    /* Now write the user data */
    nbytes = SIZEOF_MIB_USER_AP;
    src = (unsigned short *)userData;
    offset = OFFSET_USER_CONFIG;

    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }

#ifdef ECOS
    if(cyg_thread_self() != cyg_thread_idle_thread())
#endif        
        os_ExitCriticalSection;
    WatchdogEnable();
#ifdef ECOS
    if(cyg_thread_self() != cyg_thread_idle_thread())
        cyg_mutex_unlock( &writeFlash_mutex );
#endif
    return TRUE;
}

/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApResetToFactorySettings(void)
{
    int rc;

    rc = FALSE;

    memset(mFactorySettings, 0xff, sizeof(mFactorySettings));

    /* Read the factory default configuration from flash memory */
    if (ApReadFactorySettings((mib_AP_t *)mFactorySettings))
    {
        rc = ApWriteUserSettings((mib_AP_t *)mFactorySettings);
    }

    return rc;
}

/**
 * Check FW file integrity before FW upgrade
 *
 * @param pData Pointer to binary buffer
 * @param len Buffer length
 * @return 1: success
 *         0: failure
 */
static int
ApCheckImageIntegrity(char *pData, unsigned long len)
{
    UINT32 *tmp, checksum, i;
    tmp = (UINT32 *)pData;

    if (*tmp != 0x4c56524d)
    {
#ifdef AP31HOST
        if(*tmp == 0x54534f48)
        {
            *tmp = 0x4c56524d;
            return 1;
        }
        else
#endif
        {
            /*wrong signature */
            if ((int) ((UINT32 *) 0xfffffff3) == 0xffffffff)
                return 1;  /*old bootcode, let it pass */
            return 0;
        }
    }
    checksum = *tmp;
    len = *(++tmp);
    for (i = 0; i < len + 8; i += 4)
    {
        checksum += *tmp++;
    }
    checksum = ~checksum;
    /*checksum should be 0 */
    return (!checksum);
}

/**
 * Extract current boot code version
 * 
 * @return 0xaabbccdd = Numerical version of 0.0.0.0 - 9.9.9.9
 *          
 */
static UINT32
bootVersion(void)
{
    int i;
    UINT32 vers = 0;
    UINT8 *tmp = (UINT8 *) 0xfffffff4;

    for (i = 3;i >= 0; i--) {
        vers |= (*tmp - 0x30) << i*8;
        tmp += 2;
    }
    return (vers);
}

#ifdef CONDOR2
/* Private structure for ApDecapImage function */
typedef struct apImg_t {
    char *pImg[5];
    int  imgLen[5];
}apImg_t;

/**
 * Check combined image integrity and decapsulate individual binaries
 * 
 * @param pData Pointer to combined image
 * @param dataLen Length for pData
 * @param apImg Buffer to store decap'd binaries pointer and size
 *
 * @return Number of images decapsulated
 *          
 */
static int 
ApDecapImage(char *pData, unsigned long dataLen, apImg_t *apImg)
{
    UINT32 *tmp, chksum, i;
    UINT32 bootVers, len;

    if(!(pData && apImg && dataLen)) 
    {
        return 0;
    }

    tmp = (UINT32 *) pData;

    if ((*tmp != 0x4c56524d)||(dataLen != *(tmp+1))) 
    {
        return 0;
    }

    chksum = *tmp;
    len = *(++tmp);
    for (i = 0; i < len - 4; i += 4) 
    {
        chksum += *tmp++;
    }

    if (~chksum) 
    {
        return 0;
    }

    /* We need all images valid. Only single img is supported now */ 
    /* Sanity check first to support 1 img for now */
    if(((img_hdr_t *) pData)->imgCnt != 1)
    {
        memset(apImg,0,sizeof(apImg_t));
        return 0;
    }

    for(i = 0; i < ((img_hdr_t *) pData)->imgCnt; i++) 
    {
        apImg->pImg[i] = pData + ((img_hdr_t *) pData)->img[i].offset;
        apImg->imgLen[i] = *((UINT32 *)(apImg->pImg[i] + 4)) + 12; 

        /* This is for backward compatibility with old binary format */
        if(!ApCheckImageIntegrity(apImg->pImg[i],apImg->imgLen[i])) 
        {
            memset(apImg,0,sizeof(apImg_t));
            return 0;
        }
    }
    return 1; /* return imgCnt, when more bin types supported */
}
#endif /* CONDOR2 */

#ifdef VOIP
static int 
ApReadVoIPConfig(char *pData, unsigned long len)
{
    unsigned long nbytes;
    UINT16 *dst, *src;
    
    if (!flashDrvInitialized)
    {
        return FALSE;
    }

    dst = (unsigned short *)pData;
    nbytes = len;
//    src = VOIP_CONFIG_SECTOR * 0x10000 | flashbase;
    src = voip_config_sector * 0x10000 | flashbase;
    
	/* check VoIP configuration version tag to ensure data compatibility. */
    if ( *src != VOIP_CONFIG_VERSION ) // increase VOIP_CONFIG_VERSION whenever VoIP config data structure changes
    {
         *pData = NULL;
         return FALSE;
    }     

    src++;
                 
    while (nbytes)
    {
         *dst++ = *src++;
         nbytes -= sizeof(UINT16);
    }                   
    
    return TRUE;
}    

static int
ApStoreVoIPConfig(char *pData, unsigned long len)
{
    unsigned long nbytes;
    UINT16 *src;
    unsigned long offset;

    if (!flashDrvInitialized)
    {
        return FALSE;
    }
    WatchdogDisable();

//    EraseSectors(VOIP_CONFIG_SECTOR, 1);
    EraseSectors(voip_config_sector, 1);


    WatchdogEnable();
    
    if (pData == NULL)
     return FALSE;

    src = (unsigned short *)pData;
    nbytes = len;
//    offset = VOIP_CONFIG_SECTOR * 0x10000;
    offset = voip_config_sector * 0x10000;

	/* set VoIP configuration version tag to ensure data compatibility. */
    flashWriteWord(offset, VOIP_CONFIG_VERSION ); //increase VOIP_CONFIG_VERSION whenever VoIP config data structure changes

    offset += sizeof(UINT16);

    while (nbytes)
    {
         if ( !flashWriteWord(offset, *src++) )
		 {
			diag_printf("ApStoreVoIPConfig() flashWriteWord ERROR!\n");
		 }
         offset += sizeof(UINT16);
         nbytes -= sizeof(UINT16);
    }     
         
    return TRUE;
}
#endif

#ifdef VOIP
#undef 	ENABLE_BACKUP_CODE
#else
#define ENABLE_BACKUP_CODE 1 
#endif

/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */

extern int ApUpgradeFirmware(char *pData, unsigned long len)
{
    unsigned long nbytes;
    unsigned short *src;
    unsigned long offset;

    UINT32 *tmp;
    if (!flashDrvInitialized || (pData == NULL))
    {
        return FALSE;
    }
    WatchdogDisable();

#ifdef ENABLE_BACKUP_CODE

#ifndef CONDOR2
    if (!ApCheckImageIntegrity(pData, len))
    {
        WatchdogEnable();
        return FALSE;
    }
#else
    {
        apImg_t apImg;
        if ( ApDecapImage(pData,len,&apImg) == 1 )
        {
            pData = apImg.pImg[0];
            len = apImg.imgLen[0];
        } 
        else 
        {
            WatchdogEnable();
            return FALSE;
        }
    }
#endif /* CONDOR2 */

    /* only erase sector 7 - 13 for backup image */
    /* if no valid image here */
    tmp = (UINT32 *)(flashbase + 0x70000);    

    if (*tmp != 0x4c56524d)
    {
        EraseSectors(7, SECTOR_FACTORY_CONFIG - 7);

        src = (unsigned short *)pData;
        nbytes = len;
        offset = 0x70000;
        if ((int) (UINT32 *) 0xfffffff3 == 0xffffffff) /*old bootcode */
        {
            src += 8;
            nbytes -= 8;
        }
        while (nbytes)
        {
            flashWriteWord(offset, *src);
            ++src;
            offset += sizeof(UINT16);
            nbytes -= sizeof(UINT16);
        }
    }

    /*then erase 0 - 6 for upgrade image */
    EraseSectors(0, 7);

    src = (unsigned short *)pData;
    if ((int) (UINT32 *) 0xfffffff3 == 0xffffffff) /*old bootcode */
    {
        src += 8;
    }
    src += 2;     /*leave out signature for later */
    nbytes = len - 4;

    offset = 4;
    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }
    /* write the signature now */
    src = (unsigned short *)pData;
    flashWriteWord(0, *src);
    ++src;
    flashWriteWord(2, *src);

#else /* ENABLE_BACKUP_CODE*/

#ifdef VOIP
	// non backup code
    diag_printf ("FW update with backup code disabled (0, %d).\n", flashNsect-4); 
    EraseSectors(0, flashNsect-4);	
#else
    EraseSectors(0, SECTOR_FACTORY_CONFIG - 1);
#endif

    src = (unsigned short *)pData;
    nbytes = len;
    offset = 0;
    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }
#endif /* ENABLE_BACKUP_CODE*/
    WatchdogEnable();

    return TRUE;
}

extern int UpgradeFirmware(char *pData, unsigned long len)
{
    return ApUpgradeFirmware(pData, len);
}

/** public functions **/

/**
 * User supplied function to initialize Flash driver struct
 * At startup time, the system will call this function to initialize
 * the flash driver.
 * 
 * @param apfdDriver_t
 *               Pointer to Flash Driver init struct.
 *               The initialized flash driver stuct is returned.
 */
void
apfdDriverInit(apfdDriver_t *apfdDrv)
{
    cyg_mutex_init(&writeFlash_mutex);
    
    if (apfdDrv != NULL)
    {
        apfdDrv->WordMode = TRUE;
        apfdDrv->wrfunc.WrWord = flashWriteWord;
        apfdDrv->EraseSector = EraseFlashSector;
        apfdDrv->ReadFactorySettings = ApReadFactorySettings;
        apfdDrv->ReadUserSettings = ApReadUserSettings;
        apfdDrv->WriteUserSettings = ApWriteUserSettings;
        apfdDrv->ResetToFactorySettings = ApResetToFactorySettings;
        apfdDrv->UpgradeFirmware = ApUpgradeFirmware;
#ifdef VOIP
        apfdDrv->StoreVOIPConfig = ApStoreVoIPConfig;
        apfdDrv->ReadVOIPConfig = ApReadVoIPConfig;
#endif
    }

    /* get info from the flash */
    flashInterrogate();

    /* is it a manufacturer we know and have we heard of the part? */
    switch (manu)
    {
    case SST:  /* SST */
        split_sector_kb = 4;
        switch ( part ) {
        case 0x2780: /* SST39LF/VF400A */
        case 0xd7:   /* SST39LF/VF040 */
            flashsize = 512 * 1024;
            flashNsect = 8;
            makesplit(-1);
            break;
        case 0x2781:  /* SST39VF800A */
        case 0xd8:    /* SST39LF/VF080 */
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(15);
            break;
        case 0x2782:  /* SST39VF160A */
        case 0xd9:    /* SST39LF/VF160 */
        case 0x234A:  /* SST39VF1602 */
        case 0x234B:  /* SST39VF1601 */
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(-1);
            break;
        case 0x235A:  /* SST39VF3202 */
        case 0x235B:  /* SST39VF3201 */
        case 0x2783:  /* SST39VF320A */
            flashsize = 4 * 1024 * 1024;
            flashNsect = 64;
            makesplit(-1);
            break;
        case 0x236A:  /* SST39VF6402 */
        case 0x236B:  /* SST39VF6401 */
            flashsize = 8 * 1024 * 1024;
            flashNsect = 128;
            makesplit(-1);
            break;
        default:
            return ;
        }
        break;
    default:
        switch ( part ) {
        case 0x22da:  /* LV800BT */
        case 0x22d6: // F800T
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(15);
            break;
        case 0x225b:  /* LV800BB */
        case 0x2258: // F800B
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(0);
            break;
        case 0x38: // LV081
        case 0xd5: // F080
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(-1);
            break;
        case 0x22c4:  /* 160BT */
        case 0x22d2: // F160T
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(31);
#ifdef OS_VxW
            factory_sector = 30;        /* 2MB flash */
#endif
            break;
        case 0x2249:  /* 160BB */
        case 0x22d7: // LV160EB
        case 0x22d8: // F160B
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(0);
#ifdef OS_VxW
            factory_sector = 30;        /* 2MB flash */
#endif
            break;
        case 0xc8: // LV017
        case 0xad: // F016
        case 0x3d: // F017
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(-1);
            break;
        case 0x22f6: // LV320DT
        case 0x22a7: // LV320AT
            flashsize = 4 * 1024 * 1024;
            flashNsect = 64;
            makesplit(63);
            split_sector_kb = 8;
            break;
        case 0x22f9: // LV320DB
        case 0x22a8: // LV320AB
            flashsize = 4 * 1024 * 1024;
            flashNsect = 64;
            makesplit(0);
            split_sector_kb = 8;
            break;
        default:
            return ;
        }
        break;
    }

    flashbase = (0 - flashsize);

#ifdef VOIP
		voip_config_sector = flashNsect-3;
#endif

    flashDrvInitialized = TRUE;
}

void WaitForFlashReady(void)
{
#ifdef ECOS
    cyg_mutex_lock( &writeFlash_mutex );
    cyg_mutex_unlock( &writeFlash_mutex );
#endif
}

void EraseUserData(void)
{
    EraseFlashSector(SECTOR_FACTORY_CONFIG);
}
