/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* Purpose:
*    This file contains data types and routines used in accessing the
*    external station information database. This database contains
*    information on other stations (or APs) in a network. The information
*    includes the entity type (AP or station), the MAC address, the power
*    mode, and the station of the entity with respect to the station
*    containing this table.
*
* Public Procedures:
*    extStaDb_AddSta      Add a station to the table
*    extStaDb_DelSta      Delete a station from the table
*    extStaDb_SetState    Set the state of an external station
*    extStaDb_GetState    Get the state of an external station
*    extStaDb_SetPwrMode  Set the power mode of an external station
*    extStaDb_GetPwrMode  Get the power mode of an external station
*
* Notes:
*    None.
*
*****************************************************************************/

#ifndef _STADB_H_
#define _STADB_H_

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "IEEE_types.h"
#include "osif.h"
#include "host.h"
#include "sysinfo.h"
#include "host.h"
#include "pool.h"
#include "mib.h"
#include "smeMain.h"
#include "api2.h"
#include "api.h"
#include "util.h"
#include "mib.h"
#include "memmap.h"
#include "qos.h"
#include "mlme.h"
#include "ds.h"
#include "keyMgmtCommon.h"
#include "keyMgmt.h"


/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */
typedef enum
{
    LOCATE_SUCCESS,
    ADD_SUCCESS,
    DEL_SUCCESS,
    POWER_SUCCESS,
    STATE_SUCCESS,
    NOT_INITIALIZED,
    LOCATE_FAILURE,
    STATION_EXISTS_ERROR,
    TABLE_ACCESS_ERROR,
    TABLE_FULL_ERROR,
    INVALID_STATE_ERROR,
    NOT_AUTHENTICATED,
    NOT_ASSOCIATED,
    ALREADY_AUTHENTICATED,
    ALREADY_ASSOCIATED,
    ALREADY_AUTHENTICATING,
    ALREADY_ASSOCIATING,
    ALREADY_DEASSOCIATING,
    ALREADY_REASSOCIATING,
#ifdef WPA
    RSN_IE_BUF_OVERFLOW,
#endif
} extStaDb_Status_e;
/* */
/* The set of possible status responses to operations performed on the table */
/* */

typedef enum
{
    UNAUTHENTICATED = 0,
    SME_INIT_AUTHENTICATING,
    EXT_INIT_AUTHENTICATING,
    AUTHENTICATED,
    SME_INIT_DEAUTHENTICATING,
    EXT_INIT_DEAUTHENTICATING,
    SME_INIT_ASSOCIATING,
    EXT_INIT_ASSOCIATING,
    ASSOCIATED,
    SME_INIT_REASSOCIATING,
    EXT_INIT_REASSOCIATING,
    SME_INIT_DEASSOCIATING,
    EXT_INIT_DEASSOCIATING
} extStaDb_State_e;
/* */
/* The possible states a station or AP can have with respect to the */
/* station containing the external station table */
/* */

typedef enum
{
    BONLY_MODE,
    GONLY_MODE,
    MIXED_MODE,
    AONLY_MODE,
} extStaDb_ClientMode_e;

typedef enum
{
    MYFALSE,
    MYTRUE
}Boolean;

#ifdef GUEST_ACCESS
typedef enum
{
    HOME_USERS = 0,     /* have access to all services offered by the AP */
    GUEST_USERS         /* can access internet-only connection but will not 
                           access local LAN */
} extStaDb_IsGuestAccess_e;
#endif

#ifdef IEEE80211H
typedef struct basic_info_t
{
    BOOLEAN capability;
    UINT32 mToken;
    UINT8 measureStartTime[8];  /* indicates if the Sta is a QSTA */
    UINT16 measureDuration;     /* the duration in which the measurement takes place */
    UINT16 measuredChannel;     /* channel the station is currently measuring */   
    IEEEtypes_MeasurementRepMap_t mMap; /* the result of measurement report */
} basic_info;

typedef struct tcc_info_t
{
    BOOLEAN capability;
    UINT32 mToken;
    UINT8 measureStartTime[8];  /* indicates if the Sta is a QSTA */
    UINT16 measureDuration;     /* the duration in which the measurement takes place */
    UINT16 measuredChannel;     /* channel the station is currently measuring */    
} tcc_info;

typedef struct rpi_info_t
{
    BOOLEAN capability;
    UINT32 mToken;
    UINT8 measureStartTime[8];  /* indicates if the Sta is a QSTA */
    UINT16 measureDuration;     /* the duration in which the measurement takes place */
    UINT16 measuredChannel;     /* channel the station is currently measuring */   
} rpi_info;

typedef struct extStaDb_measurement_info_t
{
    UINT32 DiaglogToken;
    basic_info mBasic;
    tcc_info mTcc;
    rpi_info mRpi;    
} extStaDb_measurement_info_t;

#endif /* IEEE80211h */

typedef struct extStaDb_StaInfo_t
{
    IEEEtypes_MacAddr_t Addr;
    BOOLEAN AP;
    BOOLEAN SmeInitiated;
    extStaDb_State_e State;
    IEEEtypes_PwrMgmtMode_e PwrMode;
    UINT16 StnId;
    UINT16 Aid;
    UINT32 TimeStamp;
    UINT16 QueueToUse;
    UINT8 ClientMode;
    /* new State Machine housekeeping */
    AssocSrvSta   mgtAssoc;
    AuthRspSrvSta mgtAuthRsp;
    AuthReqSrvSta mgtAuthReq;
    PowerSaveMonitor pwrSvMon;
    keyMgmthsk_hsm_t keyMgmtHskHsm;
#ifdef STA_INFO_DB
    UINT8    Sq2;              /* Signal Quality 2 */
    UINT8    Sq1;              /* Signal Quality 1 */
    UINT8    Rate;             /* rate at which frame was received */
    UINT8    RSSI;             /* RF Signal Strength Indicator */
#endif
    UINT8  ApMode;
#ifdef QOS_FEATURE
    UINT8 IsStaQSTA; //indicates if the Sta is a QSTA
    UINT8 MoreDataAck;//Sta can preocess Ack frames with MoreDataBit in FrmCtl to 1
#endif
#ifdef IEEE80211H
    UINT16 ListenInterval;     /* Listen interveal */
    BOOLEAN IsSpectrumMgmt;    /* Is capable to do spectrum management */
    extStaDb_measurement_info_t measureInfo;
#endif /* IEEE80211H */
    keyMgmtInfo_t keyMgmtStateInfo; //keyMgmtInfo;
    //keyMgmt_StateInfo_t keyMgmtStateInfo;//keep this as the last element in this data structure
#ifdef APCFGUR
	RemoteCtrlSrv rmSrv;
	UINT8	UR;
#endif
#ifdef PPPoE_SUPPORT
	UINT16  Session_ID;
//	UINT8   IamWaiting;
#endif
#ifdef WMM_PS_SUPPORT
	QoS_WmeInfo_Info_t    Qosinfo;  /** store apsd qosinfo **/
#endif
    UINT32	aveRSSI;
#ifdef GUEST_ACCESS
    UINT8 IsGuestAccess;    /* Guest Users can access internet-only connection 
                               but will not access local LAN                  */
#endif
}
extStaDb_StaInfo_t;
/* */
/* The information recorded for an external station: */
/* 1) Its MAC address */
/* 2) Whether or not it is an AP */
/* 3) The state with respect to the station containing this database */
/* 4) The power mode of the external station */
/* */
#define AGING_TIMER_VALUE_IN_SECONDS   10 

/*============================================================================= */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

/******************************************************************************
 *
 * Name: extStaDb_Init
 *
 * Description:
 *    Routine to initial the structures used in the external stations table.
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    None.
 *
 * Return Value:
 *    Status indicating success or failure.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern WL_STATUS extStaDb_Init( UINT16 MaxStns );

/******************************************************************************
 *
 * Name: extStaDb_AddSta
 *
 * Description:
 *    This routine adds a station to the external station table.
 *
 * Conditions For Use:
 *    External station table has been initialized.
 *
 * Arguments:
 *    Arg1 (i  ): StaInfo - Pointer to a structure containing information
 *                          about the station being added
 *
 * Return Value:
 *    Status indicating the results of the operation.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern extStaDb_Status_e extStaDb_AddSta ( extStaDb_StaInfo_t *StaInfo_p);

/******************************************************************************
 *
 * Name: extStaDb_DelSta
 *
 * Description:
 *    This routine deletes a station from the external station table.
 *
 * Conditions For Use:
 *    External station table has been initialized.
 *
 * Arguments:
 *    Arg1 (i  ): Addr_p  - Pointer to the MAC address of the station to be
 *                          deleted
 *
 * Return Value:
 *    Status indicating the results of the operation.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern extStaDb_Status_e extStaDb_DelSta ( IEEEtypes_MacAddr_t *Addr_p, int option);

/******************************************************************************
 *
 * Name: extStaDb_GetStaInfo
 *
 * Description:
 *    This routine attempts to retrieve the state for the given MAC address.
 *
 * Conditions For Use:
 *    External station table has been initialized.
 *
 * Arguments:
 *    Arg1 (i  ): Addr_p  - Pointer to the MAC address of the station for
 *                          which the state is to be retrieved
 *    Arg2 (  o): StaInfo_p - Pointer to the variable that will contain the
 *                          requested station information
 *
 * Return Value:
 *    Status indicating the results of the operation.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern extStaDb_Status_e extStaDb_GetState2( IEEEtypes_MacAddr_t *Addr_p,
            extStaDb_State_e *State_p );


extern extStaDb_StaInfo_t  *extStaDb_GetStaInfo( IEEEtypes_MacAddr_t *Addr_p,
            int option);

UINT32 Wang32BitMix( UINT32 Key ) SECTION_ITCM;

#define STADB_DONT_UPDATE_AGINGTIME 0
#define STADB_UPDATE_AGINGTIME      1
#define STADB_NO_BLOCK              3


/******************************************************************************
 *
 * Name: extStaDb_SetPwrMode
 *
 * Description:
 *    This routine attempts to set the given power mode for the given MAC
 *    address.
 *
 * Conditions For Use:
 *    External station table has been initialized.
 *
 * Arguments:
 *    Arg1 (i  ): Addr_p  - Pointer to the MAC address of the station for
 *                          which a power mode update is to be made
 *    Arg2 (i  ): NewMode - The new power mode of the station
 *
 * Return Value:
 *    Status indicating the results of the operation.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern extStaDb_Status_e extStaDb_SetPwrMode(
        IEEEtypes_MacAddr_t *Addr_p,
        IEEEtypes_PwrMgmtMode_e NewMode );

/******************************************************************************
 *
 * Name: extStaDb_GetPwrMode
 *
 * Description:
 *    This routine attempts to retrieve the power mode for the given MAC
 *    address.
 *
 * Conditions For Use:
 *    External station table has been initialized.
 *
 * Arguments:
 *    Arg1 (i  ): Addr_p - Pointer to the MAC address of the station for
 *                         which the power mode is to be retrieved
 *    Arg2 (  o): Mode_p - Pointer to the variable that will contain the
 *                         requested power mode information
 *
 * Return Value:
 *    Status indicating the results of the operation.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern extStaDb_Status_e extStaDb_GetPwrMode(
        IEEEtypes_MacAddr_t *Addr_p,
        IEEEtypes_PwrMgmtMode_e *Mode_p );

extern extStaDb_Status_e extStaDb_SetAid(
        IEEEtypes_MacAddr_t *Addr_p,
        UINT16 Aid);

extern extStaDb_Status_e extStaDb_GetStnId(
        IEEEtypes_MacAddr_t *Addr_p,
        UINT16 *StnId_p );
extern UINT16 extStaDb_entries( UINT8 );
extern int set_sta_aging_time(int minutes);
extern UINT16 extStaDb_list( UINT8 *buf , UINT8);

#ifdef WPA
extern extStaDb_Status_e extStaDb_SetRSNDataTrafficEnabled(
        IEEEtypes_MacAddr_t *Addr_p, UINT8 value );
extStaDb_Status_e extStaDb_SetRSNPwk(IEEEtypes_MacAddr_t *Addr_p,
                                     UINT8* pEncryptKey, UINT32* pTxMICKey, UINT32* pRxMICKey);
extStaDb_Status_e extStaDb_SetRSNPwkAndDataTraffic(
    IEEEtypes_MacAddr_t *Addr_p,
    UINT8* pEncryptKey, UINT32* pTxMICKey, UINT32* pRxMICKey );
extStaDb_Status_e extStaDb_GetRSNPwk(IEEEtypes_MacAddr_t *Addr_p,
                                     UINT8* pEncryptKey, UINT32* pTxMICKey, UINT32* pRxMICKey);
extStaDb_Status_e extStaDb_SetRSNPmk(IEEEtypes_MacAddr_t *Addr_p, UINT8* pPMK );
extStaDb_Status_e extStaDb_GetKeyMgmtInfo(IEEEtypes_MacAddr_t *Addr_p, keyMgmtInfo_t *KeyMgmtInfo);
extStaDb_Status_e extStaDb_SetKeyMgmtInfo(IEEEtypes_MacAddr_t *Addr_p, keyMgmtInfo_t *KeyMgmtInfo);
//extStaDb_Status_e extStaDb_SetTimerData(IEEEtypes_MacAddr_t *Addr_p,
//                                        timer_Data_t *tData);
extStaDb_Status_e extStaDb_GetRSN_IE( IEEEtypes_MacAddr_t *Addr_p,
                                      UINT8 *RsnIE_p );
extStaDb_Status_e extStaDb_SetRSN_IE( IEEEtypes_MacAddr_t *Addr_p,
                                      IEEEtypes_RSN_IE_t *RsnIE_p );
extern extStaDb_Status_e extStaDb_GetPairwiseTSC( IEEEtypes_MacAddr_t *Addr_p,
            UINT32 *pTxIV32, UINT16 *pTxIV16 );
extern extStaDb_Status_e extStaDb_SetPairwiseTSC( IEEEtypes_MacAddr_t *Addr_p,
            UINT32 TxIV32, UINT16 TxIV16 );
extStaDb_Status_e extStaDb_GetStaInfoAndKeys(
    IEEEtypes_MacAddr_t *Addr_p,
    extStaDb_StaInfo_t *StaInfo_p,
    int AgingTimeMode,
    PacketType_e mode);
extStaDb_Status_e extStaDb_GetStaInfoAndTxKeys( IEEEtypes_MacAddr_t *, extStaDb_StaInfo_t *, UINT32);
extStaDb_Status_e extStaDb_GetStaInfoAndRxKeys( IEEEtypes_MacAddr_t *, extStaDb_StaInfo_t *, UINT32);
//extStaDb_Status_e extStaDb_SetKeyMgmtState(IEEEtypes_MacAddr_t *Addr_p,
//        keyMgmtState_e State);
//extStaDb_Status_e extStaDb_SetTimerData(IEEEtypes_MacAddr_t *Addr_p,
    //                                    timer_Data_t *tData);
extStaDb_Status_e extStaDb_SetPhase1Key(IEEEtypes_MacAddr_t *Addr_p,
                                        UINT16 *Phase1Key,
                                        PacketType_e mode, UINT32 RxIV32);
void extStaDb_SendGrpKeyMsgToAllSta();
void extStaDb_SetNewState4AllSta(extStaDb_State_e NewState);
extStaDb_StaInfo_t * extStaDb_GetStaInfoWPA(IEEEtypes_MacAddr_t *Addr_p,
                                           UINT32 AgingTimeMode);
extern void *itcm_malloc(UINT32 size);
extern void extStaDb_ProcessAgeEvt(void);
extern extStaDb_Status_e extStaDb_SetQoSOptn(
                                    IEEEtypes_MacAddr_t *,
                                    UINT8 );
extern UINT8 extStaDb_GetQoSOptn(IEEEtypes_MacAddr_t *Addr_p);
extern void extStaDb_RemoveAllStns(void);
extern extStaDb_Status_e extStaDb_GetWMM_DeliveryEnableInfo(
                                    IEEEtypes_MacAddr_t *,
                                    UINT8 , UINT8 *, UINT8 *);

extern UINT8 extStaDb_Check_Uapsd_Capability(
                                    IEEEtypes_MacAddr_t *);
extern UINT8 extStaDb_Check_ALL_AC_DeliveryEnableInfo(
                                    IEEEtypes_MacAddr_t *);
extern extStaDb_Status_e extStaDb_GetMeasurementInfo(
                                    IEEEtypes_MacAddr_t *,
                                    extStaDb_measurement_info_t *);
extern extStaDb_Status_e extStaDb_SetMeasurementInfo(
                                    IEEEtypes_MacAddr_t *,
                                    extStaDb_measurement_info_t *);



#endif
#endif /* _STADB_H */
