#ifndef __DHCPS_API_H__
#define __DHCPS_API_H__

/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    dhcps_api.h
 * \brief   Header file for DHCP Server Module - API
 *
 * Author(s):    Yevgeny Yanovsky
 * Date:         2004-06-22
 *
 * $Revision: #1 $
 * $Date: 2004/10/21 $
 */

#include "wltypes.h"


/* Debug */

/* #define DHCPS_debug */


/*!
 * Here defined configuration parameters for DHCP Server
 */


#define DHCPS_DEF_ADMIN_STAT            TRUE
#define DHCPS_MAX_NUM_OF_STATIONS       16
#define DHCPS_DEF_LEASE_TIME            86400       /* sec - 1 day */

/* Definitions for Task*/

#define sysinfo_DHCPS_TASK_NAME         "DHCPServer"
#define sysinfo_DHCPS_TASK_STACK_SIZE   1024*8
#define sysinfo_DHCPS_TASK_PRI          6

#define DHCPS_max_num_declined_addresses_CNS    10


/*!
 * Here defined enumerators for DHCP Server
 */

typedef enum {

    DHCPS_NO_ENTRY,
    DHCPS_STATIC_ENTRY,
    DHCPS_DYNAMIC_ENTRY

} DHCPS_STATION_TYPE;

/*!
 * Here defined structures for DHCP Server
 */

#define DHCPS_host_name_max_len_CNS 64

typedef struct _DHCPS_STATION_PROPERTIES
{
    DHCPS_STATION_TYPE  st_type;
    unsigned long       ip_address;
    unsigned char       mac_addr[6];
    unsigned long       expired_time;
    char                host_name[DHCPS_host_name_max_len_CNS];

} DHCPS_STATION_PROPERTIES;


/*!
 * Here defined API functions for DHCP Server
 */


/*!
 * GENERAL NOTE: ALL IP ADDRESSES HERE GIVEN IN HOST ORDER!!!
 */


/******************************************************************************
 *
 * Name:   dhcpsDbInit
 *
 * Description:
 *   This routine is called to initialize the DHCP Server DB.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern WL_STATUS dhcpsDbInit( void );



/******************************************************************************
 *
 * Name:   dhcpsInit
 *
 * Description:
 *   This routine is called to initialize the DHCP Server Task
 *   and related components.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern WL_STATUS dhcpsInit( void );




/******************************************************************************
 *
 * Name:   dhcpsEnable
 *
 * Description:
 *   This routine is called to change the administrative status
 *   of the DHCP Server.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): enabled    - if TRUE status set to enabled,
 *                              otherwise to disabled.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   If administrative status is disabled incoming release requests
 *      will be proceeded.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern WL_STATUS dhcpsEnable( unsigned long enabled );




/******************************************************************************
 *
 * Name:   dhcpsIsEnable
 *
 * Description:
 *   This routine is called to get the administrative status
 *   of the DHCP Server.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   TRUE if administrative status is enabled, FALSE if disabled
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern unsigned long dhcpsIsEnable( void );




/******************************************************************************
 *
 * Name:   dhcpsSetRange
 *
 * Description:
 *   This routine is called to configure the address pool range
 *   for the DHCP Server.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): start_addr - Starting IP address in the pool
 *   Arg2 (i  ): end_addr   - Ending IP address in the pool
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   Number of addresses (stations) in the range should not
 *   exceed DHCPS_MAX_NUM_OF_STATIONS, device own IP address is not counted.
 *   By default starting address in the pool is the first address on the subnet,
 *   ending address is a minimum between last address on subnet and
 *   DHCPS_MAX_NUM_OF_STATIONS not including device own IP address.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern WL_STATUS dhcpsSetRange ( unsigned long start_addr, unsigned long end_addr );




/******************************************************************************
 *
 * Name:   dhcpsGetRange
 *
 * Description:
 *   This routine is called to get the address pool range
 *   for the DHCP Server.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): p_start_addr - Pointer to storage for Starting IP address value
 *   Arg2 (i  ): p_end_addr   - Pointer to storage for Ending IP address value
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void dhcpsGetRange ( unsigned long *p_start_addr, unsigned long *p_end_addr );




/******************************************************************************
 *
 * Name:   dhcpsSetStation
 *
 * Description:
 *   This routine is called to configure the static address for
 *   specific station.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): ip_addr    - IP address to be assigned
 *   Arg2 (i  ): mac_addr   - MAC Address of the station
 *   Arg3 (i  ): delete    - remove entry if TRUE
 *
 * Return Value:
 *   Status indicating success or failure.
 *
 * Notes:
 *   If station with given IP already exist as dynamic entry it
 *   will be changed to static, if it already exists as static entry -
 *   its MAC address will be updated
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern WL_STATUS dhcpsSetStation ( unsigned long ip_addr,
                                   unsigned char mac_addr [],
                                   unsigned long delete );




/******************************************************************************
 *
 * Name:   dhcpsGetStation
 *
 * Description:
 *   This routine is called to get static or dynamic entry
 *                          and its properties from the DB.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): ip_addr        - IP Address as key for requested entry
 *   Arg2 (i  ): p_properties    - pointer to the storage for
 *                                the properties of the requested entry
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void dhcpsGetStation ( unsigned long ip_addr,
                                   DHCPS_STATION_PROPERTIES *p_properties );




/******************************************************************************
 *
 * Name:   dhcpsGetBulkStations
 *
 * Description:
 *   This routine is called to get all entries from DB
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): p_num_of_st  - pointer to the storage for the number of stations
 *   Arg2 (i  ): stationsTbl   - array of station properties structures
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   Size of table should be according to DHCPS_MAX_NUM_OF_STATIONS
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void dhcpsGetBulkStations ( unsigned long *p_num_of_st,
                                   DHCPS_STATION_PROPERTIES stationsTbl[]);




/******************************************************************************
 *
 * Name:   dhcpsGetNumAllocStation
 *
 * Description:
 *   This routine is called to get number allocated addresses from
 *                          dynamic pool and number of remained free
 *                          addresses which can be allocated.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1(i  ): p_num_alloc  - pointer to the storage for
 *                             the number of allocated addresses
 *   Arg2 (i  ): p_num_free  - pointer to the storage for
 *                             the number of free addresses
 *
 * Return Value:
 *   None
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void dhcpsGetNumAllocStation ( unsigned long *p_num_alloc,
                                      unsigned long *p_num_free);




/******************************************************************************
 *
 * Name:   dhcpsSetLeaseTime
 *
 * Description:
 *   This routine is called to configure the lease time
 *   for the DHCP Server.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): lease_time - lease time in seconds
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern WL_STATUS dhcpsSetLeaseTime ( unsigned long lease_time );





/******************************************************************************
 *
 * Name:   dhcpsGetLeaseTime
 *
 * Description:
 *   This routine is called to get the address pool range
 *   for the DHCP Server.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): p_lease_time - Pointer to storage for lease time value
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void dhcpsGetLeaseTime ( unsigned long *p_lease_time );





/******************************************************************************
 *
 * Name:   dhcpsSetIpAddress
 *
 * Description:
 *   This routine is called to notify the DHCP Server about
 *   new IP Address and subnet mask configured in the system.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): ip_address - IP Address configured in the system
 *   Arg2 (i  ): mask       - subnet mask configured in the system
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void dhcpsSetIpAddress ( unsigned long ip_address, unsigned long mask );







/******************************************************************************
 *
 * Name:   dhcpsSetDNS
 *
 * Description:
 *   This routine is called to notify the DHCP Server about
 *   DNS Servers IP addresses in the system.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 (i  ): prim_dns_addr - Primary DNS Server IP address
 *   Arg2 (i  ): sec_dns_addr  - Secondary DNS Server IP address
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void dhcpsSetDNS ( unsigned long prim_dns_addr, unsigned long sec_dns_addr);





#endif /* __DHCPS_API_H__ */
