#ifndef _ETHERSWITCH_H_
#define _ETHERSWITCH_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */ 
/*!
 * \file    etherswitch.h
 * \brief   This file contains the function prototypes and definitions for the 
 *          6060 Ethernet Switch (L2 switch)
 */

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================
#define etherSwitch_MSG_RCVD               (1<<0)
#define etherSwitch_TIMER_EXPIRED          (1<<1)

/*-------------------------------------*/
/* Ethernet Switch Task Messages       */
/*-------------------------------------*/
typedef struct etherSwitchQ_Msg_t
{
    UINT32 command;
    UINT32 data;
}
etherSwitchQ_Msg_t;

/*----------------*/
/* Timer Messages */
/*----------------*/
typedef struct etherSwitchQ_TimerMsg_t
{
    UINT32 *PendingData_p;
    UINT32 Id;
}
etherSwitchQ_TimerMsg_t;
//
// Structure used for messages placed on the Ethernet Task's
// timer message queue. The fields in the structure are:
// 1) PendingData_p - Pointer to relevant saved data
// 2) Id            - ID of the timer
//

//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================

typedef enum
{
   ETHSW_PORT_PRI_LOWEST,
   ETHSW_PORT_PRI_LOW,
   ETHSW_PORT_PRI_HIGH,
   ETHSW_PORT_PRI_HIGHEST,

} ETHSW_PORT_PRIORITY; /* DefPri of Reg 0x07 */

typedef enum
{
   ETHSW_PORT_INGRESS_LIMIT_ALL,
   ETHSW_PORT_INGRESS_LIMIT_BC_MC_FL,
   ETHSW_PORT_INGRESS_LIMIT_BC_MC,
   ETHSW_PORT_INGRESS_LIMIT_BC,

} ETHSW_PORT_INGRESS_LIMIT_MODE; /* LimitMode of Reg 0x0A */

typedef enum
{
   ETHSW_PORT_RATE_NOLIMITED,
   ETHSW_PORT_RATE_128K,
   ETHSW_PORT_RATE_256K,
   ETHSW_PORT_RATE_512K,
   ETHSW_PORT_RATE_1M,
   ETHSW_PORT_RATE_2M,
   ETHSW_PORT_RATE_4M,
   ETHSW_PORT_RATE_8M,

} ETHSW_PORT_LIMIT_RATE; /* Pri3Rate, Pri2Rate, Pri1Rate, Pri0Rate, EgressRate of Reg 0x0A */

UINT8 EthSw_Port_SetPriority(UINT32 port, ETHSW_PORT_PRIORITY pri);
UINT8 EthSw_Port_SetFlowCtrl(UINT32 port, UINT32 enable);
UINT8 EthSw_Port_SetIngressRateLimitMode(UINT32 port, ETHSW_PORT_INGRESS_LIMIT_MODE mode);
UINT8 EthSw_Port_SetIngressLimitRate(UINT32 port, ETHSW_PORT_LIMIT_RATE rate);
UINT8 EthSw_Port_SetEgressLimitRate(UINT32 port, ETHSW_PORT_LIMIT_RATE rate);

/******************************************************************************
 * 
 * Name: etherSwitch_Init  
 * 
 * Description: 
 *   This routine is called to initialize the ethernet switch registers and 
 *   ports.
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
#ifdef AP5X
extern void etherSwitch_Init( int wanPort );
#else
extern void etherSwitch_Init( void );
#endif
/******************************************************************************
 * 
 * Name: etherSwitchTask_Start 
 * 
 * Description: 
 *   This routine is called to start the Ethernet Switch Management Task running. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues has been initialized by calling 
 *   etherSwitch_Init(). 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None.
 * 
 *****************************************************************************/
extern void etherSwitchTask_Start( void );

/******************************************************************************
 * 
 * Name: etherSwitchTask_SendEvent 
 * 
 * Description: 
 *   This routine provides the mechanism to trigger an event indicating to 
 *   the Ethernet Switch Task that a message has been written to its 
 *   queue. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues has been initialized by calling 
 *   etherSwitchTask_Init(). 
 * 
 * Arguments: 
 *   Arg1 (i  ): Event - value indicating which queue has received a message 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 * 
 *****************************************************************************/
extern WL_STATUS etherSwitchTask_SendEvent( UINT32 Event );


/******************************************************************************
 *
 * Name: etherSwitchQ_WriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where ethernet
 *   switch messages are placed for the Ethernet Switch Service Task. If
 *   writing to the queue cannot immediately occur, then the routine returns
 *   with a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling etherSwitchQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): Msg_p - a pointer to the message to be placed on
 *                       the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS etherSwitchQ_WriteNoBlock( etherSwitchQ_Msg_t *Msg_p );


/******************************************************************************
 * 
 * Name: etherSwitchReset  
 * 
 * Description: 
 *   This routine is resets the ethernet switch using the GPIO port.
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 * END PDL 
 * 
 *****************************************************************************/
extern void etherSwitchReset( void );

#endif /* _ETHERSWITCH_H_ */
