/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
 *
 * Purpose:
 *    This file contains the implementations of the function prototypes given
 *    in the associated header file for the buffer pool module.
 *
 * Public Procedures:
 *    pool_Init               Initialzie the pool module
 *    pool_Create             Create a buffer pool
 *    pool_GetBuf             Get a buffer from a pool
 *    pool_FreeBuf            Return a buffer back to the pool
 *    pool_GetMinNumBufs      The least number of buffers free at any
 *                            point in time for a pool
 *    pool_GetCurrentNumBufs  The current number of free buffers in a pool
 *
 *
 * Notes:
 *    Buffer pools are allocated using the pool_CREATE macro in the header
 *    file. The allocation reserves memory for the number of buffers
 *    specified; the memory includes not just the buffers that will be used
 *    by the requestor, but also structures that allow for the buffers to be
 *    chained together in a list - the buffers are actually a component of
 *    these structures. After the allocation, the pool must still be setup
 *    by calling the pool_Create() routine - this is the routine that ties
 *    the buffers allocated together in a list. Once this is accomplished,
 *    a requestor for buffers calls the pool_GetBuf() routine and then
 *    returns them to the pool by calling the pool_FreeBuf() routine.
 *
 *****************************************************************************/

#ifndef _POOL_H_
#define _POOL_H_

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

//=============================================================================
//                            PUBLIC DEFINITIONS
//=============================================================================
#define pool_ALIGNMENT 16


//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================
typedef void* pool_ID_t;
   //
   // Type used for a pool ID
   //

typedef struct pool_BufHdr_t
{
   char                  Rsvd[pool_ALIGNMENT -
                              (sizeof(char)+2*(sizeof(char*)))];
   char                  UsageCnt;
   struct pool_CtlBlk_t *Pool_p;
   struct pool_Buf_t    *NextFreeBuf_p;
}pool_BufHdr_t;
   //
   // The buffer header; it contains:
   // 1) A field to align the structure according to that specified by
   //    pool_ALIGNMENT
   // 2) The number of usage instances of the buffer; it is allowed that
   //    the buffer be used by more than one client or more than one copy
   //    can be used by a single client; when this count is decreased
   //    to zero, the buffer can be freed (returned to the pool)
   // 3) A pointer to the pool the buffer is associated with
   // 4) A pointer that allows for chaining the buffer together with other
   //    buffers in the pool
   //

typedef struct pool_Buf_t
{
   pool_BufHdr_t  BufHdr;
   char           BufBody[1];
}pool_Buf_t;
   //
   // The structure used that contains both the buffer header and the
   // buffer itself. These structures are what gets allocated in the
   // pool_CREATE macro.
   //

typedef struct pool_CtlBlk_t
{
   char        PoolDescrStr[16];
   pool_Buf_t *FreeList_p;
   UINT16      NumPoolBufs;
   UINT16      NumFreeBufs;
   UINT16      MinNumFreeBufs;
   UINT16      BufBodyLen;
   pool_Buf_t *StartFreeList_p;
}pool_CtlBlk_t;
   //
   // The pool structure control block; it contains:
   // 1) The string containing the name of the pool
   // 2) A pointer to the first free buffer in the pool
   // 3) The total number of buffers in the pool (unallocated +
   //    allocated)
   // 4) The number of buffers in the pool that can currently be allocated
   // 5) A statistic that keeps track of the low water mark of the minimum
   //    number of free buffers
   // 6) The length of the buffers that are allocated after a request
   //


//=============================================================================
//                             PUBLIC MACROS
//=============================================================================
#ifndef AP_SW
#define  pool_CREATE( PoolName,                                        \
                      PoolDescrStr,                                    \
                      BufLen,                                          \
                      NumBufs )                                        \
                                                                       \
   static struct                                                       \
   {                                                                   \
      char  Rsvd[pool_ALIGNMENT];                                      \
      struct                                                           \
      {                                                                \
         pool_BufHdr_t  BufHdr;                                        \
         char           BufBody[BufLen];                               \
         char           Rsvd[pool_ALIGNMENT -                          \
                             ((sizeof(pool_BufHdr_t) + BufLen) %       \
                             pool_ALIGNMENT)];                         \
      } ActualBuffers[NumBufs];                                        \
}  PACK PoolName ## Bfrs __attribute__ ((section (".sqbuf")));                                           \
                                                                       \
   static pool_CtlBlk_t PoolName __attribute__ ((section (".sqbuf"))) = { {PoolDescrStr},                   \
                                     (pool_Buf_t*) & PoolName ## Bfrs, \
                                     NumBufs,                          \
                                     NumBufs,                          \
                                     NumBufs,                          \
                             BufLen ,                       
                                     (pool_Buf_t*) & PoolName ## Bfrs};
  
#else
#define  pool_CREATE( PoolName,                                        \
                      PoolDescrStr,                                    \
                      BufLen,                                          \
                      NumBufs )                                        \
                                                                       \
   static struct                                                       \
   {                                                                   \
      char  Rsvd[pool_ALIGNMENT];                                      \
      struct                                                           \
      {                                                                \
         pool_BufHdr_t  BufHdr;                                        \
         char           BufBody[BufLen];                               \
         char           Rsvd[pool_ALIGNMENT -                          \
                             ((sizeof(pool_BufHdr_t) + BufLen) %       \
                             pool_ALIGNMENT)];                         \
      } ActualBuffers[NumBufs];                                        \
}PoolName ## Bfrs ;         \
                                                                       \
   static pool_CtlBlk_t PoolName = { {PoolDescrStr},                   \
                                     (pool_Buf_t*) & PoolName ## Bfrs, \
                                     NumBufs,                          \
                                     NumBufs,                          \
                                     NumBufs,                          \
                               BufLen ,     (pool_Buf_t*) & PoolName ## Bfrs};
#endif
   //
   // The CREATE macro accepts the following parameters:
   // 1) PoolName - a C variable used to reference the pool
   // 2) PoolDescStr - a string describing the pool
   // 3) BufLen - the length of the buffers allocated to the pool
   // 4) NumBufs - the number of buffers allocated to the pool
   //
   // The macro defines a structure and associated variable that is
   // uniquely named. The structure is based upon the supplied buffer
   // length and the number of buffers; the variable name is based
   // upon PoolName, which has also been supplied. The structure defines
   // the header and body used for buffers, is aligned according to
   // pool_ALIGNMENT, and is used to allocate the requested number of
   // buffers - thus, it defines the buffer pool. The associated variable
   // creates an instance of the structure and this is how the allocation
   // actually occurs. Finally, a variable that is a pool control block
   // is declared; the free list pointer field in the control block is
   // set to point to the variable that defines the allocated buffer pool.
   // Note that this variable is cast as a buffer type; work still needs
   // to be done so that the free list pointer points to the buffers
   // allocated within the pool, and not have it point to the beginning
   // of the structure containing the pool. This is taken care of in
   // the pool_Create() routine, which chains together the allocated
   // buffers on to the free list and also performs alignment by
   // making sure the allocated buffers begin along the specified byte
   // boundaries.
   //


#ifdef AP_SW
#define  sqpool_CREATE( PoolName,                                        \
                      PoolDescrStr,                                    \
                      BufLen,                                          \
                      NumBufs )                                        \
                                                                       \
   static struct                                                       \
   {                                                                   \
      char  Rsvd[pool_ALIGNMENT];                                      \
      struct                                                           \
      {                                                                \
         pool_BufHdr_t  BufHdr;                                        \
         char           BufBody[BufLen];                               \
         char           Rsvd[pool_ALIGNMENT -                          \
                             ((sizeof(pool_BufHdr_t) + BufLen) %       \
                             pool_ALIGNMENT)];                         \
      } ActualBuffers[NumBufs];                                        \
}PoolName ## Bfrs __attribute__ ((section (".sqbuf")));         \
                                                                       \
   static pool_CtlBlk_t PoolName = { {PoolDescrStr},                   \
                                     (pool_Buf_t*) & PoolName ## Bfrs, \
                                     NumBufs,                          \
                                     NumBufs,                          \
                                     NumBufs,                          \
           BufLen ,      (pool_Buf_t*) & PoolName ## Bfrs};
#define  sdpool_CREATE( PoolName,                                        \
                      PoolDescrStr,                                    \
                      BufLen,                                          \
                      NumBufs )                                        \
                                                                       \
   static struct                                                       \
   {                                                                   \
      char  Rsvd[pool_ALIGNMENT];                                      \
      struct                                                           \
      {                                                                \
         pool_BufHdr_t  BufHdr;                                        \
         char           BufBody[BufLen];                               \
         char           Rsvd[pool_ALIGNMENT -                          \
                             ((sizeof(pool_BufHdr_t) + BufLen) %       \
                             pool_ALIGNMENT)];                         \
      } ActualBuffers[NumBufs];                                        \
}PoolName ## Bfrs __attribute__ ((section (".sdbuf")));         \
                                                                       \
   static pool_CtlBlk_t PoolName = { {PoolDescrStr},                   \
                                     (pool_Buf_t*) & PoolName ## Bfrs, \
                                     NumBufs,                          \
                                     NumBufs,                          \
                                     NumBufs,                          \
                             BufLen ,    (pool_Buf_t*) & PoolName ## Bfrs};


#ifdef D_CACHE //add  by laser for GST
#define  sd_cache_pool_CREATE( PoolName,                                        \
                      PoolDescrStr,                                    \
                      BufLen,                                          \
                      NumBufs )                                        \
                                                                       \
   static struct                                                       \
   {                                                                   \
      char  Rsvd[pool_ALIGNMENT];                                      \
      struct                                                           \
      {                                                                \
         pool_BufHdr_t  BufHdr;                                        \
         char           BufBody[BufLen];                               \
         char           Rsvd[pool_ALIGNMENT -                          \
                             ((sizeof(pool_BufHdr_t) + BufLen) %       \
                             pool_ALIGNMENT)];                         \
      } ActualBuffers[NumBufs];                                        \
}PoolName ## Bfrs __attribute__ ((section (".sdcache")));         \
                                                                       \
   static pool_CtlBlk_t PoolName = { {PoolDescrStr},                   \
                                     (pool_Buf_t*) & PoolName ## Bfrs, \
                                     NumBufs,                          \
                                     NumBufs,                          \
                                     NumBufs,                          \
                             BufLen ,    (pool_Buf_t*) & PoolName ## Bfrs};

#endif

#endif

//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================

/******************************************************************************
 * Name: pool_Init
 *
 * Description:
 *    Initialize the pool module
 *
 * Conditions for Use:
 *    Called during system initialization.
 *
 * Arguments:
 *   Arg1 (i  ): StartOfMem_p - Pointer to the start of available memory
 *   Arg2 (i  ): EndOfMem_p   - Pointer to the end of available memory
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern WL_STATUS pool_Init( char *StartOfMemory_p, char *EndOfMemory_p );


/******************************************************************************
 * Name: pool_Create
 *
 * Description:
 *    Create and initialize a pool of buffers
 *
 * Conditions for Use:
 *    The pool module has been initialized and the pool has been allocated.
 *
 * Arguments:
 *   Arg1 (i  ): PoolCtlBlk_p - Pointer to the allocated pool structure
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern pool_ID_t pool_Create ( pool_CtlBlk_t *PoolCtlBlk_p );


/******************************************************************************
 * Name: pool_GetBuf
 *
 * Description:
 *    Get a buffer from the pool, if available
 *
 * Conditions for Use:
 *    The pool module has been initialized and the pool created.
 *
 * Arguments:
 *    Arg1 (i  ): PoolId - Id of the pool to get a buffer from
 *
 * Return Value:
 *    A pointer to a buffer
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern char* pool_GetBuf ( pool_ID_t PoolId );


/******************************************************************************
 * Name: pool_FreeBuf
 *
 * Description:
 *    Return a buffer to the pool.
 *
 * Conditions for Use:
 *    The pool module has been initialized, the pool created, and the buffer
 *    has been allocated.
 *
 * Arguments:
 *    Arg1 (i  ): ReturnedBuf_p - Pointer to the buffer being returned
 *
 * Return Value:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
//extern WL_STATUS pool_FreeBuf ( char* ReturnedBuf_p );


/******************************************************************************
 * Name:   pool_MakeVirtualBufCopy
 *
 * Description:
 *    Routine to make a virtual copy of an existing buffer. This simply
 *    amounts to the usage count of the buffer being incremented - hence,
 *    calls to pool_FreeBuf do not actually free the buffer until the
 *    usage count is down to zero.
 *
 * Conditions for Use:
 *    The pool module has been initialized and the pool has been created
 *
 * Arguments:
 *    Arg1 (i  ): Buf_p - Pointer to the buffer to make a virtual copy of
 *
 * Return Value:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern void pool_MakeVirtualBufCopy ( char *Buf_p );


/******************************************************************************
 * Name: pool_GetMinNumBufs
 *
 * Description:
 *    Returns a statistic giving the least number of free buffers that the
 *    pool was down to at any time.
 *
 * Conditions for Use:
 *    The pool module has been initialized.
 *
 * Arguments:
 *    Arg1 (i  ): PoolId - The ID of the pool for which the statistic is
 *                         being asked
 *
 * Return Value:
 *    The smallest number of free buffers that has occurred.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern UINT32 pool_GetMinNumBufs ( pool_ID_t PoolId );


/******************************************************************************
 * Name: pool_GetCurrentNumBufs
 *
 * Description:
 *    Returns a statistic giving the current number of free buffers that
 *    are available out of a given pool.
 *
 * Conditions for Use:
 *    The pool module has been initialized.
 *
 * Arguments:
 *    Arg1 (i  ): PoolId - The ID of the pool for which the statistic is
 *                         being asked
 *
 * Return Value:
 *    The current number of free buffers that are available.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
extern UINT32 pool_GetCurrentNumBufs ( pool_ID_t PoolId );
extern WL_STATUS pool_FreeBuf ( char* ReturnedBuf_p );



#endif
