/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* Purpose:
*    This file contains the function prototypes and definitions for the
*    Station Management Entity (SME) Main module.
*
* Public Procedures:
*    smeMain_Init       Initialzies the SME Task and related components
*    smeMain_Start      Starts running the SME Task
*    smeMain_SendEvent  Mechanism used to trigger an event indicating a
*                           message has been received on one of the message
*                           queues
*
* Notes:
*    None.
*
*****************************************************************************/

#ifndef _SMEMAIN_H_
#define _SMEMAIN_H_

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "pool.h"
#include "IEEE_types.h"
#ifdef STA
#include "host.h"
#endif
#include "osif.h"


/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */
#define smeMain_MGMT_MSG_RCVD      (1<<0)
#define smeMain_CMD_RCVD           (1<<1)
#define smeMain_ERROR_RCVD         (1<<2)

/*------------------------------------------------------------*/
/* The types of problems that can be reported to the SME Task */
/*------------------------------------------------------------*/
typedef enum
{
    SMEQ_TX_ERROR,
    SMEQ_BCN_LOST
} smeQ_Error_e;

typedef UINT16 smeQ_TxError_t;
typedef UINT16 smeQ_BcnError_t;

/*-----------------------------------------------------------------*/
/* Management messages coming from the MAC Management Service Task */
/*-----------------------------------------------------------------*/
typedef struct smeQ_MgmtMsg_t
{
    IEEEtypes_SmeNotify_e MsgType;
    union
    {
        IEEEtypes_PwrMgmtCfrm_t PwrMgmtCfrm;
        IEEEtypes_ScanCfrm_t ScanCfrm;
        IEEEtypes_JoinCfrm_t JoinCfrm;
        IEEEtypes_AuthCfrm_t AuthCfrm;
        IEEEtypes_AuthInd_t AuthInd;
        IEEEtypes_DeauthCfrm_t DeauthCfrm;
        IEEEtypes_DeauthInd_t DeauthInd;
        IEEEtypes_AssocCfrm_t AssocCfrm;
        IEEEtypes_AssocInd_t AssocInd;
        IEEEtypes_ReassocCfrm_t ReassocCfrm;
        IEEEtypes_ReassocInd_t ReassocInd;
        IEEEtypes_DisassocCfrm_t DisassocCfrm;
        IEEEtypes_DisassocInd_t DisassocInd;
        IEEEtypes_ResetCfrm_t ResetCfrm;
        IEEEtypes_StartCfrm_t StartCfrm;
#ifdef IEEE80211H
        IEEEtypes_MRequestInd_t MrequestInd;
        IEEEtypes_MRequestCfrm_t MrequestCfrm;
        IEEEtypes_MeasureCfrm_t MeasureCfrm;
        IEEEtypes_MReportCfrm_t MreportCfrm;
        IEEEtypes_MReportInd_t MreportInd;
        IEEEtypes_ChannelSwitchCfrm_t ChannelSwitchCfrm;
        IEEEtypes_ChannelSwitchInd_t ChannelSwitchInd;
        IEEEtypes_TPCAdaptCfrm_t TPCAdaptCfrm;
#endif /* IEEEtypes_MRequestInd_t */
    } Msg;
}
smeQ_MgmtMsg_t ;

/*--------------------------------*/
/* Commands from the CB Processor */
/*--------------------------------*/
typedef struct smeQ_HostCmd_t
{
    host_MsgHdr_t MsgHdr;
    union
    {
        host_getHwSpec_t HwSpec;
        host_802_11SetWep_t SetWep;
        APCmd_DS_802_11_QUERY_STATUS QueryStatus;
        APCmd_CMD_802_11_STA_LIST StaList;
        APCmd_CMD_802_11_GET_AUTH_STA_LIST AuthStaList;
        APCmd_CMD_802_11_GET_ASSOC_STA_LIST AssocStaList;
        APCmd_CMD_802_11_QUERY_STA_INFO StaInfo;
        APCmd_CMD_802_11_SSID SsId;
        APCmd_CMD_802_11_PRIVACY_OPTION PrivOption;
        APCmd_CMD_802_11_AUTHENTICATION_MODE AuthMode;
        APCmd_CMD_802_11_FRAGMENTATION_THRESHOLD FragThresh;
        APCmd_CMD_802_11_RTS_THRESHOLD RtsThreshold;
        APCmd_CMD_802_11_RTS_RETRY_LIMIT RetryLimit;
        APCmd_DS_802_11_DATA_RATE DataRate;
        APCmd_DS_802_11_RADIO_PREAMBLE RadioPreamble;
        APCmd_DS_802_11_BEACON_PERIOD BeaconPeriod;
        APCmd_DS_802_11_DTIM_PERIOD DtimPeriod;
        APCmd_DS_802_3_GET_STAT GetStat;
        APCmd_DS_802_3_LINK_SPEED LinkSpeed;
        APCmd_DS_802_3_IP_ADDRESS IpAddr;
        APCmd_DS_802_3_SUBNET_MASK SubnetMask;
        APCmd_DS_802_3_GATEWAY Gateway;
        APCmd_DS_802_3_DHCP Dhcp;
        APCmd_DS_802_3_ICMP Icmp;
        APCmd_CMD_802_11_LINK_TEST LinkTest;
    } Msg;
}
smeQ_HostCmd_t;

/*--------------------------------------------------------------------*/
/* Problems notifications - either transmit errors reported by the    */
/* 802.11 MAC Messages Transmitted Task or a beacon lost notification */
/* reported by the MAC Management Service Task.                       */
/*--------------------------------------------------------------------*/
typedef struct smeQ_ErrorMsg_t
{
    smeQ_Error_e ErrorType;
    union
    {
        smeQ_TxError_t TxErrors;
        smeQ_BcnError_t BcnError;
    } Msg;
}
smeQ_ErrorMsg_t;


/*============================================================================= */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */


/******************************************************************************
 *
 * Name: smeMain_Init
 *
 * Description:
 *   This routine is called to initialize the the SME Task and related
 *   components.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS smeMain_Init( void );

/******************************************************************************
 *
 * Name: smeMain_Start
 *
 * Description:
 *   This routine is called to start the SME Task running.
 *
 * Conditions For Use:
 *   The task and its associated queues have been initialized by calling
 *   smeMain_Init().
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void smeMain_Start( void );

/******************************************************************************
 *
 * Name: smeMain_SendEvent
 *
 * Description:
 *   This routine provides the mechanism to trigger an event indicating to
 *   the SME Task that a message has been written to one of its queues.
 *
 * Conditions For Use:
 *   The task and its associated queues has been initialized by calling
 *   smeMain_Init().
 *
 * Arguments:
 *   Arg1 (i  ): Event - value indicating which queue has received a message
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS smeMain_SendEvent( UINT8 Event );

extern WL_STATUS smeQ_MgmtWriteNoBlock( smeQ_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: smeQ_CbProcWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where messages
 *   from the CB Processor Task are placed for the SME Task. If writing
 *   to the queue cannot immediately occur, then the routine returns with
 *   a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): CbCmd_p - a pointer to the message to be placed on the
 *                         queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
#ifdef STA
extern WL_STATUS smeQ_HostCmdWriteNoBlock( smeQ_HostCmd_t *HostCmd_p );
#endif
/******************************************************************************
 *
 * Name: smeQ_ErrorWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message that indicates a system fault
 *   has occured to the appropriate queue for the SME Task. If writing to
 *   the queue cannot immediately occur, then the routine returns with a
 *   failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): ErrorMsg_p - a pointer to the message to be placed on
 *                            the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS smeQ_ErrorWriteNoBlock( smeQ_ErrorMsg_t *ErrorMsg_p);
extern smeQ_MgmtMsg_t *smeQ_GetMgmtBuf(void);
#ifdef STA
extern smeQ_HostCmd_t *smeQ_GetHostCmdBuf(void);
#endif
WL_STATUS SendDeauthCmd(IEEEtypes_MacAddr_t *MacAddr_p);
UINT32 SendStartCmd(void);
void SendResetCmd(int);
#endif /* _SMEMAIN_H_ */

