/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* Purpose:
*    This file contains the function prototypes and definitions for the
*    Tx 80211 Main module.
*
* Public Procedures:
*    Tx80211_Init       Initialzies the SME Task and related components
*    Tx80211_Start      Starts running the SME Task
*    Tx80211_SendEvent  Mechanism used to trigger an event indicating a
*                           message has been received on one of the message
*                           queues
*
* Notes:
*    None.
*
*****************************************************************************/

#ifndef _TX80211_H_
#define _TX80211_H_
#include "rx80211.h"

//=============================================================================
//                               INCLUDE FILES
//=============================================================================


//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================
#define tx80211_MGMT_MSG_RCVD      (1<<0)
#define tx80211_DATA_RCVD          (1<<1)
#define tx80211_CFP_START_RCVD     (1<<2)
#define tx80211_CFP_END_RCVD       (1<<3)
#define tx80211_FRAME_TX_DONE      (1<<4)
#define tx80211_INTERNAL_MSG_RCVD       (1<<5)
#define tx80211_TX_WD_TIMEOUT      (1<<6)
#define tx80211_ETHER_TX_DONE       (1<<7)
#define tx80211_PKT_AGING_TIMEOUT_RCVD  (1<<8)
#define tx80211_MemErr (1<<9)
#define tx80211_SI_TIMOUT_RECVD     (1<<10)
#define tx80211_RA_TIMOUT_RECVD     (1<<11)
#define tx80211_RA_TIMOUT_RECVD_UR  (1<<12)

#ifdef WMM_PS_SUPPORT
#define tx80211_EOSP_RECVD     (1<<13)
#endif


#define PKT_AGING_TIME 5



#define TX_PARAM_SHORT_PREAMBLE  0x4
#define TX_PARAM_LONG_PREAMBLE   0x0
#define TX_PARAM_UNICAST         0x0
#define TX_PARAM_MCBC            0x40
#define TX_PARAM_NO_MCBC         0x00
#define TX_PARAM_MCBC_MASK       0x40
#define TX_PARAM_PREAMBLE        0x4
/* b & g definitions */
#define TX_PARAM_SEND_RTS       0x0020
#define TX_PARAM_DONOT_SEND_RTS 0x0000
#define TX_PARAM_SEND_RTS_MASK  0x0020

//#define TX_PARAM_SEND_CTS_TO_SELF 0x0080

#define TX_PARAM_SELF_CTS       0x0080
#define TX_PARAM_NO_SELF_CTS 0x0000
#define TX_PARAM_SELF_CTS_MASK  0x0080


#define TX_PARAM_CCK            0x0100
#define TX_PARAM_OFDM           0x0000
#define TX_PARAM_CCK_OFDM_MASK  0x0100

#define TX_PARAM_LISTEN_MODE_MASK  0x0600
#define TX_PARAM_LISTEN_MODE_CCK   0x0200
#define TX_PARAM_LISTEN_MODE_OFDM  0x0400
#define TX_PARAM_LISTEN_MODE_CCK_OR_OFDM  0x0600
#define TXINFO_ETHER_BUF   2
#define TXINFO_DATA_BUF     1
#define TXINFO_ONLY_BUF     0
#ifdef WDS_FEATURE
#define TX_PARAM_WDS             0x1
#endif // WDS_FEATURE
#define TX_PARAM_ANT1   0x0008



extern UINT8 RateIndex1Mbit;
extern UINT8 RateIndex2Mbit;
extern UINT8 RateIndex11Mbit;
extern UINT8 StopWirelessflag;


typedef struct tx80211_MgmtMsg_t
{
    UINT32 reserved;
    UINT16 stnId;
    UINT16 macQId;
    macmgmtQ_MgmtMsg_t MgmtFrame;
}
PACK_END tx80211_MgmtMsg_t;

typedef struct tx80211_Frame_t
{
    SwInfo_t SwInfo;
    IEEEtypes_Frame_t Frame;
}
PACK_END tx80211_Frame_t;


typedef struct tx_pktbuf_t
{
    txInfo_t TxInfo;
    SwInfo_t SwInfo;
    IEEEtypes_DataFrame_t Frame;
}
PACK_END tx_pktbuf_t;

//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================


/******************************************************************************
 *
 * Name: tx80211_Init
 *
 * Description:
 *   This routine is called to initialize the the SME Task and related
 *   components.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS tx80211_Init( void );

/******************************************************************************
 *
 * Name: tx80211_Start
 *
 * Description:
 *   This routine is called to start the SME Task running.
 *
 * Conditions For Use:
 *   The task and its associated queues have been initialized by calling
 *   smeMain_Init().
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void tx80211_Start( void );

/******************************************************************************
 *
 * Name: tx80211_SendEvent
 *
 * Description:
 *   This routine provides the mechanism to trigger an event indicating to
 *   the SME Task that a message has been written to one of its queues.
 *
 * Conditions For Use:
 *   The task and its associated queues has been initialized by calling
 *   smeMain_Init().
 *
 * Arguments:
 *   Arg1 (i  ): Event - value indicating which queue has received a message
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS tx80211_SendEvent( UINT32 Event );

/******************************************************************************
 *
 * Name: tx80211_MgmtWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where IEEE 802.11
 *   management messages are placed for the SME Task. If writing to the
 *   queue cannot immediately occur, then the routine returns with a failure
 *   status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): MgmtMsg_p - a pointer to the message to be placed on
 *                           the queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS tx80211Q_MgmtWriteNoBlock( tx80211_MgmtMsg_t *MgmtMsg_p );

/******************************************************************************
 *
 * Name: tx80211_DataWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where messages
 *   from the CB Processor Task are placed for the SME Task. If writing
 *   to the queue cannot immediately occur, then the routine returns with
 *   a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): CbCmd_p - a pointer to the message to be placed on the
 *                         queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS tx80211Q_DataWriteNoBlock( txInfo_t *Data_p );
extern txInfo_t* tx80211_AllocTxInfoBuf(UINT32 type);
extern void tx80211_InitTxInfo(txInfo_t *txInfo_p, UINT8 shortpreamble);
extern int DoWepAndTxForWlanRx(txInfo_t *txInfo_p);
extern void UseHWTkipEngineForEtherRx(txInfo_t *txInfo_p, UINT8 *pSrc,UINT8 *pTxMICKey,keyMgmtInfo_t *pKeyMgmtInfo,UINT8 *pEncrKey);
extern int UseHWTKIPAndTxForWlanRx(txInfo_t *txInfo_p,  
                                   UINT8 *pTxMICKey,keyMgmtInfo_t *pKeyMgmtInfo,UINT8 *pEncrKey);

void DoCCMPAndTxForEtherRx(txInfo_t *txInfo_p, UINT8 *pEncrKey,
                           keyMgmtInfo_t *pKeyMgmtInfo, UINT8 *pSrc);
extern int DoCCMPAndTxForWlanRx(txInfo_t *txInfo_p, UINT8 *pEncrKey,
                                keyMgmtInfo_t *pKeyMgmtInfo,UINT8);

#endif /* _TX80211_H_ */

